@@ -4,7 +4,7 @@ This is the base multidimensional array class which implements all
 structural array operations but treats the array contents as opaque
 objects
 
-$Id: generic.py,v 1.52 2004/06/28 21:00:03 jaytmiller Exp $
+$Id: generic.py,v 1.54 2004/08/12 21:27:21 jaytmiller Exp $
 """
 
 import types as _types
@@ -207,7 +207,7 @@ class NDArray(_ndarray):
             else:
                 self._bytestride = bytestride
 
-            if buffer:
+            if buffer is not None:
                 self._data = buffer
             else:
                 size = self._bytestride * self.nelements()
@@ -1009,8 +1009,10 @@ def _concat(arrs):
     except TypeError:
         dest = arrs[0]._clone(shape=destShape)
     else:
-        dest  = arrs[0].__class__(shape=destShape, type=convType)
-        
+        try:
+            dest  = arrs[0].__class__(shape=destShape, type=convType)
+        except TypeError:
+            dest  = arrs[0].__class__(shape=destShape)        
     ix = 0
     for a in arrs:
         if a._shape[1:] != rShape:
