@@ -1,4 +1,4 @@
-"""$Id: memmap.py,v 1.25 2004/02/26 18:56:00 jaytmiller Exp $
+"""$Id: memmap.py,v 1.26 2004/08/25 10:13:25 jaytmiller Exp $
 
 This module implements a class wrapper (Memmap) around the
 mmap type which adds some additional properties.
@@ -24,9 +24,9 @@ file might be (a) much bigger and (b) already existing.   Other file modes
 >>> m
 <Memmap on file 'memmap.tst' with mode='w+', length=48, 0 slices>
 
-In this case, file is initialized to 0.  As in C stdio, when a file is
-opened for 'w+', it is truncated (effectively deleted) if it already
-exists.  The initial contents of a w+ Memmap are undefined.
+As in C stdio, when a file is opened for 'w+', it is truncated
+(effectively deleted) if it already exists.  The initial contents of a
+w+ Memmap are undefined.
 
 Slice m into the buffers "n" and "p" which will correspond to numarray:
 
@@ -43,7 +43,7 @@ NOTE: You *can not* make overlapping slices of a Memmap:
 >>> q = m[20:28]
 Traceback (most recent call last):
 ...
-IndexError: Slice overlaps previous slice of same file.
+IndexError: Slice overlaps prior slice of same file.
 
 NOTE: you *can not* make an array directly from a Memmap:
 
@@ -343,7 +343,7 @@ class Memmap:
                   begin < b < end or 
                   begin < e < end or 
                   b == begin and e == end and b != e):
-                raise IndexError("Slice overlaps previous slice of same file.")
+                raise IndexError("Slice overlaps prior slice of same file.")
 
     def __len__(self):
         """len(self) is the number of bytes in Memmap 'self'.
@@ -552,7 +552,7 @@ class Memmap:
         >>> t=m.insert(45, 100)
         Traceback (most recent call last):
         ...
-        IndexError: Slice overlaps previous slice of same file.
+        IndexError: Slice overlaps prior slice of same file.
         >>> m.flush()
         >>> len(m)
         700
@@ -749,7 +749,7 @@ class MemmapSlice:
     def flush(self):
         """flush(self)
         """
-        raise ValueError("Only the 'root' Memmap should be flushed.")
+        raise TypeError("Only the 'root' Memmap should be flushed.")
 
     def _chkIndex(self, i, End=0):
         """_chkIndex(self, i) raises an IndexError if 'i' is not a valid
