@@ -58,7 +58,7 @@ del key, value
 Py2NumType = {
     bool:  _nt.Bool,
     int:   _nt.Long,
-    long:  _nt.Long,
+    long:  _nt.Int64,
     float: _nt.Float,
     complex: _nt.Complex
     }
@@ -112,7 +112,9 @@ def _storePyValueListInBuffer(buffer, Ctype, valuelist):
 
 def _fillarray(size, start, delta, type=None):
     ptype = _maxtype((start, delta))
-    if PyINT_TYPES.has_key(ptype):
+    if ptype == long:
+        ptype = _nt.Int64
+    elif PyINT_TYPES.has_key(ptype):
         ptype = _nt.Long
     elif PyREAL_TYPES.has_key(ptype):
         ptype = _nt.Float
@@ -177,12 +179,16 @@ def _frontseqshape(seq):
     """Find the length of all the first elements, return as a list"""
     if not len(seq):
         return (0,)
+    if isinstance(seq, types.StringType):
+        return (len(seq),)
     try:
         shape = []
         while 1:
             shape.append(len(seq))
             try:
                 seq = seq[0]
+                if isinstance(seq, types.StringType):
+                    return shape
             except IndexError:
                 return shape
     except TypeError:
@@ -309,8 +315,11 @@ def array(sequence=None, typecode=None, copy=1, savespace=0,
             else:
                 a = sequence
         else:
-            type = getTypeObject(sequence, type, typecode) 
-            a = sequence.astype(type) # make a re-typed copy
+            type = getTypeObject(sequence, type, typecode)
+            if not copy and type is sequence.type():
+                a = sequence
+            else:
+                a = sequence.astype(type) # make a re-typed copy
         if shape is not None:
             a.setshape(shape)
         return a
@@ -664,7 +673,7 @@ class NumArray(_numarray._numarray, _gen.NDArray, UsesOpPriority):
         """returns 1 iff an array is c-contiguous, aligned, and not
         byteswapped."""
         return (self.iscontiguous() and self.isaligned() and not
-                self.isbyteswapped())
+                self.isbyteswapped()) != 0
 
     def is_f_array(self):
         """returns 1 iff an array is fortan-contiguous, aligned, and not
@@ -672,11 +681,13 @@ class NumArray(_numarray._numarray, _gen.NDArray, UsesOpPriority):
         return (self.is_fortran_contiguous() and self.isaligned() and not
                 self.isbyteswapped())
 
-    def new(self, type):
+    def new(self, type=None):
         """Return a new array of given type with same shape as this array
 
         Note this only creates the array; it does not copy the data.
         """
+        if type is None:
+            type = self._type
         return self.__class__(shape=self._shape, type=type)
         
     def type(self):
@@ -738,7 +749,8 @@ class NumArray(_numarray._numarray, _gen.NDArray, UsesOpPriority):
             return operand.__add__(self)
         else:
             r = ufunc.add(operand, self)
-            r.__class__ = self.__class__
+            if isinstance(r, NumArray):
+                r.__class__ = self.__class__
             return r
 
     def __sub__(self, operand):
@@ -754,7 +766,8 @@ class NumArray(_numarray._numarray, _gen.NDArray, UsesOpPriority):
             return operand.__sub__(self)
         else:
             r = ufunc.subtract(operand, self)
-            r.__class__ = self.__class__
+            if isinstance(r, NumArray):
+                r.__class__ = self.__class__
             return r
 
     def __mul__(self, operand):
@@ -770,7 +783,8 @@ class NumArray(_numarray._numarray, _gen.NDArray, UsesOpPriority):
             return operand.__mul__(self)
         else:
             r = ufunc.multiply(operand, self)
-            r.__class__ = self.__class__
+            if isinstance(r, NumArray):
+                r.__class__ = self.__class__
             return r
 
     def __div__(self, operand):
@@ -800,7 +814,8 @@ class NumArray(_numarray._numarray, _gen.NDArray, UsesOpPriority):
             return operand.__div__(self)
         else:
             r = ufunc.divide(operand, self)
-            r.__class__ = self.__class__
+            if isinstance(r, NumArray):
+                r.__class__ = self.__class__
             return r
 
     def __rtruediv__(self, operand):
@@ -809,7 +824,8 @@ class NumArray(_numarray._numarray, _gen.NDArray, UsesOpPriority):
             return operand.__truediv__(self)
         else:
             r = ufunc.true_divide(operand, self)
-            r.__class__ = self.__class__
+            if isinstance(r, NumArray):
+                r.__class__ = self.__class__
             return r
 
     def __rfloordiv__(self, operand):
@@ -818,7 +834,8 @@ class NumArray(_numarray._numarray, _gen.NDArray, UsesOpPriority):
             return operand.__floordiv__(self)
         else:
             r = ufunc.floor_divide(operand, self)
-            r.__class__ = self.__class__
+            if isinstance(r, NumArray):
+                r.__class__ = self.__class__
             return r
 
     def __mod__(self, operand):
@@ -834,7 +851,8 @@ class NumArray(_numarray._numarray, _gen.NDArray, UsesOpPriority):
             return operand.__mod__(self)
         else:
             r = ufunc.remainder(operand, self)
-            r.__class__ = self.__class__
+            if isinstance(r, NumArray):
+                r.__class__ = self.__class__
             return r
 
     def __divmod__(self,operand):
@@ -852,7 +870,8 @@ class NumArray(_numarray._numarray, _gen.NDArray, UsesOpPriority):
             return operand.__divmod__(self)
         else:
             r = ufunc.divide_remainder(operand, self)
-            r.__class__ = self.__class__
+            if isinstance(r, NumArray):
+                r.__class__ = self.__class__
             return r
 
     def __pow__(self, operand):
@@ -868,7 +887,8 @@ class NumArray(_numarray._numarray, _gen.NDArray, UsesOpPriority):
             return operand.__pow__(self)
         else:
             r = ufunc.power(operand, self)
-            r.__class__ = self.__class__
+            if isinstance(r, NumArray):
+                r.__class__ = self.__class__
             return r
 
     def __and__(self, operand):
@@ -884,7 +904,8 @@ class NumArray(_numarray._numarray, _gen.NDArray, UsesOpPriority):
             return operand.__and__(self)
         else:
             r = ufunc.bitwise_and(operand, self)
-            r.__class__ = self.__class__
+            if isinstance(r, NumArray):
+                r.__class__ = self.__class__
             return r
 
     def __or__(self, operand):
@@ -900,7 +921,8 @@ class NumArray(_numarray._numarray, _gen.NDArray, UsesOpPriority):
             return operand.__or__(self)
         else:
             r = ufunc.bitwise_or(operand, self)
-            r.__class__ = self.__class__
+            if isinstance(r, NumArray):
+                r.__class__ = self.__class__
             return r
 
     def __xor__(self, operand):
@@ -916,7 +938,8 @@ class NumArray(_numarray._numarray, _gen.NDArray, UsesOpPriority):
             return operand.__xor__(self)
         else:
             r = ufunc.bitwise_xor(operand, self)
-            r.__class__ = self.__class__
+            if isinstance(r, NumArray):
+                r.__class__ = self.__class__
             return r
 
     def __rshift__(self, operand):
@@ -932,7 +955,8 @@ class NumArray(_numarray._numarray, _gen.NDArray, UsesOpPriority):
             return operand.__rshift__(self)
         else:
             r = ufunc.rshift(operand, self)
-            r.__class__ = self.__class__
+            if isinstance(r, NumArray):
+                r.__class__ = self.__class__
             return r
 
     def __lshift__(self, operand):
@@ -948,7 +972,8 @@ class NumArray(_numarray._numarray, _gen.NDArray, UsesOpPriority):
             return operand.__lshift__(self)
         else:
             r = ufunc.lshift(operand, self)
-            r.__class__ = self.__class__
+            if isinstance(r, NumArray):
+                r.__class__ = self.__class__
             return r
 
     # augmented assignment operators
@@ -997,11 +1022,11 @@ class NumArray(_numarray._numarray, _gen.NDArray, UsesOpPriority):
         ufunc.bitwise_xor(self, operand, self)
         return self
     
-    def __irshift(self, operand):
+    def __irshift__(self, operand):
         ufunc.rshift(self, operand, self)
         return self
     
-    def __ilshift(self, operand):
+    def __ilshift__(self, operand):
         ufunc.lshift(self, operand, self)
         return self
 
@@ -1060,7 +1085,9 @@ class NumArray(_numarray._numarray, _gen.NDArray, UsesOpPriority):
             return w
         else:
             self.swapaxes(axis,-1)
-            return self._argsort()
+            w = self._argsort()
+            w.swapaxes(axis, -1)
+            return w
 
     def argsort(self, axis=-1):
         """returns the indices of 'self' which if taken from self would return
@@ -1483,9 +1510,40 @@ def tensormultiply(array1, array2):
 
     where xxx, yyy denote the rest of the a and b dimensions.
     """
+    array1, array2 = asarray(array1), asarray(array2)
     if array1.shape[-1] != array2.shape[0]:
         raise ValueError, "Unmatched dimensions"
     shape = array1.shape[:-1] + array2.shape[1:]
     return _gen.reshape(dot(_gen.reshape(array1, (-1, array1.shape[-1])),
                             _gen.reshape(array2, (array2.shape[0], -1))), shape)
 
+def kroneckerproduct(a,b):
+	'''Computes a otimes b where otimes is the Kronecker product operator.
+	
+	Note: the Kronecker product is also known as the matrix direct product
+	or tensor product.  It is defined as follows for 2D arrays a and b
+	where shape(a)=(m,n) and shape(b)=(p,q):
+	c = a otimes b  => cij = a[i,j]*b  where cij is the ij-th submatrix of c.
+	So shape(c)=(m*p,n*q).
+
+	>>> print kroneckerproduct([[1,2]],[[3],[4]])
+	[[3 6]
+	 [4 8]]
+	>>> print kroneckerproduct([[1,2]],[[3,4]])
+	[ [3 4 6 8]]
+	>>> print kroneckerproduct([[1],[2]],[[3],[4]])
+	[[3]
+	 [4]
+	 [6]
+	 [8]]
+	'''
+	a, b = asarray(a), asarray(b)
+	if not (len(shape(a))==2 and len(shape(b))==2):
+        	raise ValueError, 'Input must be 2D arrays.'
+	if not a.iscontiguous():
+		a = _gen.reshape(a, a.shape)
+	if not b.iscontiguous():
+		b = _gen.reshape(b, b.shape)
+	o = outerproduct(a,b)
+	o.shape = a.shape + b.shape
+	return _gen.concatenate(_gen.concatenate(o, axis=1), axis=1)
