@@ -2348,6 +2348,22 @@ Bugfix dot(1D, 1D) --> scalar
 >>> dot([1,2,3],[4,5,6])
 32
 
+Bugfix bad transpose of second argument to dot
+>>> a = array([[1., 0., 0.], [0., 1., 0.], [0., 0., 1.]])
+>>> b = array([[0.5, 0.8, 0.], [-0.8, 0.5, 0.], [0., 0., 1.]])
+>>> dot(a, b)
+array([[ 0.5,  0.8,  0. ],
+       [-0.8,  0.5,  0. ],
+       [ 0. ,  0. ,  1. ]])
+>>> a
+array([[ 1.,  0.,  0.],
+       [ 0.,  1.,  0.],
+       [ 0.,  0.,  1.]])
+>>> b
+array([[ 0.5,  0.8,  0. ],
+       [-0.8,  0.5,  0. ],
+       [ 0. ,  0. ,  1. ]])
+
 Rank 0 numarray, i.e. empty-shape --> scalar
 >>> zeros(())
 array(0)
@@ -2836,11 +2852,11 @@ From bug #690640 (irregular list bug)
 >>> array([3, [4, 5]])
 Traceback (most recent call last):
 ...
-error: setArrayFromSequence: invalid sequence.
+ValueError: Nested sequences with different lengths.
 >>> array([[1,2,3], [4, 5]])
 Traceback (most recent call last):
 ...
-error: setArrayFromSequence: nested sequences of different lengths
+ValueError: Nested sequences with different lengths.
 
 Fortran version of contiguity method
 
@@ -3175,6 +3191,74 @@ Traceback (most recent call last):
 ...
 IndexError: too many slices for array shape
 
+>>> a = arange(10)
+>>> b = array(a, copy=0)
+>>> a is b
+1
+>>> b = array(a, copy=1)
+>>> a is b
+0
+>>> b = array(a, 'l', copy=0)
+>>> a is b
+1
+>>> b = array(a, 'f', copy=0)
+>>> a is b
+0
+>>> b = array(a, 'i', copy=1)
+>>> a is b
+0
+>>> b = array(a, 'f', copy=1)
+>>> a is b
+0
+
+SF# [ 995287 ] float * array fails?
+
+>>> array(0.0)+4.0*array(1.0)
+4.0
+
+SF# [ 980750 ] argmax fails on on slices of larger arrays
+
+>>> a = array([[0,1,2], [3,2,1]])
+>>> argmax(a[0]) # OK
+2
+>>> argmax(a[1]) # Should be 0
+0
+>>> argmin(a[1])
+2
+>>> argmin(array(a[1]))
+2
+
+SF# [ 1008462 ] searchsorted bug and fix
+
+>>> a = ones(100)
+>>> searchsorted(a, 1)
+0
+
+SF#
+
+>>> print kroneckerproduct([[1,2]],[[3],[4]])
+[[3 6]
+ [4 8]]
+>>> print kroneckerproduct([[1,2]],[[3,4]])
+[[3 4 6 8]]
+>>> print kroneckerproduct([[1],[2]],[[3],[4]])
+[[3]
+ [4]
+ [6]
+ [8]]
+
+
+# Large array check for blocking
+
+>>> a = arange(1001**2, shape=(1001,1001))
+>>> add.reduce(a)
+array([501000500, 501001501, 501002502, ..., 501999498, 502000499,
+       502001500])
+>>> a.byteswap()
+>>> a.togglebyteorder()
+>>> add.reduce(a)
+array([501000500, 501001501, 501002502, ..., 501999498, 502000499,
+       502001500])
 """
 from numarrayall import *
 import sys, string, cPickle
