@@ -3,9 +3,9 @@ import numerictypes as _nt
 import generic as _gen
 import memory
 import strings as chararray
-import sys, os, re, types
+import sys, os, re, types, stat
 
-__version__ = '$Revision: 1.16 $ $Date: 2004/05/12 17:03:07 $'
+__version__ = '$Revision: 1.16.2.1 $ $Date: 2004/11/16 23:02:39 $'
 
 class Char:
     """ data type Char class"""
@@ -219,7 +219,10 @@ def fromfile(file, formats, shape=-1, names=None, byteorder=sys.byteorder, align
     if isinstance(file, types.StringType):
         name = 1
         file = open(file, 'rb')
-    size = os.path.getsize(file.name) - file.tell()
+    try:
+        size = os.fstat(file.fileno())[stat.ST_SIZE] - file.tell()
+    except:
+        size = os.path.getsize(file.name) - file.tell()
 
     dummy = array(None, formats=formats, shape=0, aligned=aligned)
     itemsize = dummy._itemsize
