@@ -1,4 +1,4 @@
-# $Id: strings.py,v 1.23 2004/06/22 20:09:08 jaytmiller Exp $
+# $Id: strings.py,v 1.28.4.1 2004/11/16 23:14:50 jaytmiller Exp $
 #
 """
 Large chararray test
@@ -43,6 +43,9 @@ import types
 import os
 import re
 import arrayprint
+import warnings
+import numerictypes as _nt
+import numinclude as _numinclude
 
 _globals, _locals  = globals, locals
 
@@ -66,6 +69,10 @@ def NewCharArrayType(itemsize):
     return CharArrayTypeCache[itemsize]
 
 
+class PrecisionWarning(UserWarning):
+    pass
+
+
 class RawCharArray(num.NDArray):
     """RawCharArray(buffer=None, shape=None, byteoffset=0, bytestride=None)
 
@@ -509,12 +516,14 @@ class RawCharArray(num.NDArray):
         l.sort()
         self[:] = fromlist(l)
 
-    def argsort(self):
+    def argsort(self, axis=-1):
         """
         >>> a=fromlist(["other","that","this","another"])
         >>> a.argsort()
         array([3, 0, 1, 2])
         """
+        if axis != -1:
+            raise TypeError("CharArray.argsort() does not support the axis parameter.")
         ax = range(len(self))
         ax.sort(lambda x,y,z=self: cmp(z[x],z[y]))
         return num.array(ax)
@@ -595,6 +604,8 @@ class RawCharArray(num.NDArray):
 
     def eval(self):
         """eval(self) converts CharArray 'self' into a NumArray.
+        This is the original slow implementation based on a Python loop
+        and the eval() function.
 
         >>> array([["1","2"],["3","4"]]).eval()
         array([[1, 2],
@@ -608,8 +619,12 @@ class RawCharArray(num.NDArray):
         n.setshape(self._shape)
         return n
 
-    def fasteval(self, globals=None, locals=None, type=num.Float64):
-        """fasteval(self) converts CharArray 'self' into a NumArray.
+    def fasteval(self, type=num.Float64):
+
+        """fasteval(self, type=Float64) converts CharArray 'self' into
+        a NumArray of the specified type.  fasteval() can't convert
+        complex arrays at all, and loses precision when converting
+        UInt64 or Int64.
 
         >>> array([["1","2"],["3","4"]]).fasteval(type=num.Long)
         array([[1, 2],
@@ -620,13 +635,16 @@ class RawCharArray(num.NDArray):
         ...    pass
         """
         n = num.array(shape=self._shape, type=num.Float64)
-        _chararray.Eval((eval, globals or _globals(), locals or _locals),
-                        self, n);
+        type = _nt.getType(type)
+        _chararray.Eval((), self, n);
         if type != num.Float64:
+	    if ((type is num.Int64) or 
+		(_numinclude.hasUInt64 and type is num.UInt64)):
+                warnings.warn("Loss of precision converting to 64-bit type.  Consider using eval().", PrecisionWarning)
             return n.astype(type)
         else:
             return n
-
+        
 class CharArray(RawCharArray):
     """
     >>> array("thisthatthe othe",shape=(4,),itemsize=4)
@@ -1085,8 +1103,21 @@ def num2char(n, format, itemsize=32):
     CharArray(['0', '1', '2', '3', '4'])
     >>> num2char(num.arange(5), "%02d")
     CharArray(['00', '01', '02', '03', '04'])
+
+    Limitations:
+    
+    1. When formatted values are too large to fit into strings of
+    length itemsize, the values are truncated, possibly losing
+    significant information.
+
+    2. Complex numbers are not supported.
+    
     """
     n = num.asarray(n)
+
+    if isinstance(n.type(), _nt.ComplexType):
+        raise NotImplementedError("num2char doesn't support complex types yet.")
+
     if n.type() == num.Float64:
         wnum = n
     else:
