@@ -1,6 +1,6 @@
 """NumArray Universal Function Module
 
-$Id: ufunc.py,v 1.104 2004/06/25 13:41:01 jaytmiller Exp $
+$Id: ufunc.py,v 1.108 2004/08/12 20:44:32 jaytmiller Exp $
 """
 
 import types
@@ -214,6 +214,13 @@ def _nIOArgs(ucfuncs):
             "for operator: "+operator)
     return max(ninputs), max(noutputs)
 
+if _PROTOTYPE:
+    def _getBlockingParameters(shape, niter, overlap=0):
+        try:
+            return _blockingParametersCache[ (shape, niter, overlap) ]
+        except KeyError:
+            return _getBlockingMiss(shape, niter, overlap)
+
 def _getBlockingMiss(shape, niter, overlap=0):
     """Figure out how to split the input array into subnumarray
     close (but less than) the size of the block buffers.
@@ -232,11 +239,6 @@ def _getBlockingMiss(shape, niter, overlap=0):
          the shape of the leftover subarray
     """
     
-    # Now in C:
-    # retval = _blockingParametersCache.get((shape,niter,overlap))
-    # if retval is not None:
-    #     return retval
-    
     if shape == ():
         retval = 0, (1, (), 0, None)
         _blockingParametersCache[(shape,niter,overlap)] = retval
@@ -249,10 +251,16 @@ def _getBlockingMiss(shape, niter, overlap=0):
         # special overlap handling in this case
         nblocks, leftover = divmod(csize, niter-overlap)
         nblocks, leftover = int(nblocks), int(leftover)
-        if leftover:
-            retval = ndim-1, (nblocks, (niter,), 1, (leftover,))
+        if overlap:
+            if leftover:
+                retval = ndim-1, (nblocks, (niter,), 1, (leftover-1,))
+            else:
+                retval = ndim-1, (nblocks-1, (niter,), 1, (niter-1,))
         else:
-            retval = ndim-1, (nblocks, (niter,), 0, None)
+            if leftover:
+                retval = ndim-1, (nblocks, (niter,), 1, (leftover,))
+            else:
+                retval = ndim-1, (nblocks, (niter,), 0, None)
     else:
         for i in xrange(2,ndim+1):
             csize *= shape[-i]
@@ -433,6 +441,10 @@ class _UFunc(_ufunc._ufunc):
         return _nc.array([self._identity])
 
     if _PROTOTYPE:
+        def _cache_flush(self):
+            # print "flushing cache for", self
+            self._cache = {}
+
         def __call__(self, *args):
             raise NotImplementedError("__call__ is not implemented by base UFunc class")
 
@@ -532,8 +544,8 @@ class _BufferPool:
         if buffersize != oldsize:
             _ufunc.setBufferSize(buffersize)
             # delete existing (wrong-sized) buffers
-            self.buffers = []
             flush_caches()
+            self.buffers = []
         return oldsize
 
 _bufferPool = _BufferPool()
