@@ -94,9 +94,8 @@ Py_Correlate1d(PyObject *obj, PyObject *args)
 	PyObject   *okernel, *odata, *ocorrelated=Py_None;
 	PyArrayObject *kernel, *data, *correlated;
 
-	if (!PyArg_ParseTuple(args, "OO|O", &okernel, &odata, &ocorrelated))
-		return PyErr_Format(_Error, 
-				    "Correlate1d: Invalid parameters.");
+	if (!PyArg_ParseTuple(args, "OO|O:Correlate1d", &okernel, &odata, &ocorrelated))
+		return NULL;
 
 	/* Align, Byteswap, Contiguous, Typeconvert */
 	kernel	= NA_InputArray(okernel, tFloat64, C_ARRAY);
@@ -212,10 +211,9 @@ Py_Correlate2d(PyObject *obj, PyObject *args, PyObject *kw)
 	int      mode = PIX_NEAREST;
 	char       *keywds[] = { "kernel", "data", "output", "mode", "cval", NULL };
 
-	if (!PyArg_ParseTupleAndKeywords(args, kw, "OO|Oid", keywds, 
-		&okernel, &odata, &ocorrelated, &mode, &cval))
-		return PyErr_Format(_Error, 
-				    "Correlate2d: Invalid parameters.");
+	if (!PyArg_ParseTupleAndKeywords(args, kw, "OO|Oid:Correlate2d", keywds, 
+					 &okernel, &odata, &ocorrelated, &mode, &cval))
+		return NULL;
 
 	if ((mode < PIX_NEAREST) || (mode > PIX_CONSTANT))
 		return PyErr_Format(PyExc_ValueError,
@@ -228,7 +226,7 @@ Py_Correlate2d(PyObject *obj, PyObject *args, PyObject *kw)
 	correlated = NA_OptionalOutputArray(ocorrelated, tFloat64, C_ARRAY, 
 					    data);
 
-	if (!kernel || !data || !correlated)
+	if (!kernel || !data || !correlated)		
 		return PyErr_Format(_Error,
 				    "Correlate2d: error normalizing array inputs.");
 
@@ -280,10 +278,9 @@ Py_Shift2d(PyObject *obj, PyObject *args, PyObject *kw)
 	char          *keywds[] = { "data", "dx", "dy", 
 				    "output", "mode", "cval", NULL };
 
-	if (!PyArg_ParseTupleAndKeywords(args, kw, "Oii|Oid", keywds,
+	if (!PyArg_ParseTupleAndKeywords(args, kw, "Oii|Oid:Shift2d", keywds,
 			 &odata, &dx, &dy, &ooutput, &mode, &cval))
-		return PyErr_Format(_Error, 
-				    "Shift2d: Invalid parameters.");
+		return NULL;
 
 	if ((mode < PIX_NEAREST) || (mode > PIX_CONSTANT))
 		return PyErr_Format(PyExc_ValueError,
@@ -569,9 +566,13 @@ Py_Boxcar2d(PyObject *obj, PyObject *args, PyObject *kw)
 		return PyErr_Format(_Error,
 		    "Boxcar2d: data and output numarray need identical shapes.");
 
-	if ((kcols <=0) || (krows <= 0) || 
-	    (kcols > data->dimensions[1]) || (krows > data->dimensions[0]))
-		return PyErr_Format(_Error, "Boxcar2d: bad boxcar dimensions.");
+	if ((kcols <=0) || (krows <= 0))
+		return PyErr_Format(PyExc_ValueError,
+				    "Boxcar2d: invalid data shape.");
+	if ((kcols > data->dimensions[1]) || (krows > data->dimensions[0]))
+		return PyErr_Format(PyExc_ValueError, 
+				    "Boxcar2d: boxcar shape incompatible with"
+				    " data shape.");
 
 	Boxcar2d(krows, kcols, data->dimensions[0], data->dimensions[1], 
 		 NA_OFFSETDATA(data), NA_OFFSETDATA(output), mode, cval);
