@@ -843,6 +843,9 @@ array(data = %(data)s,
         "Set item described by index. If value is masked, mask those locations."
         if self is masked: 
             raise MAError, 'Cannot alter the masked element.'
+        # Numarray indexes rank-0 arrays with () as an extension of the
+        # idea that any index is a tuple of coordinates.  Since a rank-0
+        # array has no dimensions, the coordinate tuple is empty.
         if self._data.rank == 0:
             index = ()
         if value is masked:
@@ -876,6 +879,8 @@ array(data = %(data)s,
             if self._mask is None:
                 self._mask = make_mask_none(self._data.shape)
                 self._shared_mask = 0
+            else:
+                self.unshare_mask()
             self._mask[i:j] = 1
             return
         m = getmask(value)
@@ -889,6 +894,8 @@ array(data = %(data)s,
             if self._mask is None:
                 self._mask = make_mask_none(self._data.shape)
                 self._shared_mask = 0
+            else:
+                self.unshare_mask()
             self._mask[i:j] = m
 
     def __len__ (self):
