@@ -118,6 +118,7 @@ def convolve(input, weights, output = None, mode = "nearest", cval = 0.0,
         if not weights.shape[ii] & 1:
             origin[ii] -= 1
     footprint = numarray.abs(weights) > 1e-15
+    
     return _nd_image.correlate(input, weights, footprint, output, mode,
                                cval, origin, output_type)
 
@@ -616,6 +617,10 @@ def prewitt(input, axis = -1, output = None, mode = "nearest", cval = 0.0,
     """
     inplace = isinstance(output, numarray.NumArray)
     input = numarray.asarray(input)
+    if axis < 0:
+        axis += input.rank
+    if axis < 0 or axis >= input.rank:
+        raise RuntimeError, 'invalid axis specification'
     if inplace:
         correlate1d(input, [1, 0, -1], axis, output, mode, cval, 0,
                     output_type)
@@ -642,6 +647,10 @@ def sobel(input, axis= - 1, output = None, mode = "nearest", cval = 0.0,
     """
     inplace = isinstance(output, numarray.NumArray)
     input = numarray.asarray(input)
+    if axis < 0:
+        axis += input.rank
+    if axis < 0 or axis >= input.rank:
+        raise RuntimeError, 'invalid axis specification'
     if inplace:
         correlate1d(input, [1, 0, -1], axis, output, mode, cval, 0,
                     output_type)
@@ -656,8 +665,8 @@ def sobel(input, axis= - 1, output = None, mode = "nearest", cval = 0.0,
         return output
 
 
-def generic_laplace(input, output = None, mode = "nearest", cval = 0.0,
-                    output_type = None, derivative2 = None):
+def generic_laplace(input, derivative2, output = None,
+                    mode = "nearest", cval = 0.0, output_type = None):
     """Calculate a multidimensional laplace filter using the provided
     second derivative function.
 
@@ -699,8 +708,8 @@ def laplace(input, output = None, mode = "nearest", cval = 0.0,
         return correlate1d(input, [1, -2, 1], axis, output, mode, cval, 0,
                            output_type)
 
-    return generic_laplace(input, output, mode, cval, output_type,
-                           derivative2)
+    return generic_laplace(input, derivative2, output, mode, cval,
+                           output_type)
 
 
 def gaussian_laplace(input, sigma, output = None, mode = "nearest",
@@ -724,13 +733,13 @@ def gaussian_laplace(input, sigma, output = None, mode = "nearest",
         order[axis] = 2
         return gaussian_filter(input, sigma, order, output, mode, cval,
                                output_type)
-    return generic_laplace(input, output, mode, cval, output_type,
-                           derivative2)
+    return generic_laplace(input, derivative2, output, mode, cval,
+                           output_type)
 
 
-def generic_gradient_magnitude(input, output = None, mode = "nearest",
-                               cval = 0.0, output_type = None,
-                               derivative = None):
+def generic_gradient_magnitude(input, derivative, output = None,
+                               mode = "nearest", cval = 0.0,
+                               output_type = None):
     """Calculate a gradient magnitude using the provdide function for
     the gradient.
 
@@ -782,8 +791,8 @@ def gaussian_gradient_magnitude(input, sigma, output = None,
         return gaussian_filter(input, sigma, order, output, mode, cval,
                                output_type)
     
-    return generic_gradient_magnitude(input, output, mode, cval,
-                                      output_type, derivative)
+    return generic_gradient_magnitude(input, derivative, output, mode,
+                                      cval, output_type)
                                       
 def generic_filter1d(input, function, filter_size, axis = -1,
                      output = None, mode = "nearest", cval = 0.0,
