@@ -34,6 +34,11 @@ import _nd_image
 import filters
 import morphology
 
+def _center_is_true(structure, origin):
+    structure = numarray.array(structure)
+    coor = tuple([oo + ss // 2 for ss, oo in zip(structure.shape, origin)])
+    return bool(structure[coor])
+
 def iterate_structure(structure, iterations, origin = None):
     """Iterate a structure by dilating it with itself.
 
@@ -80,8 +85,78 @@ def generate_binary_structure(rank, connectivity):
     return output <= connectivity
 
 
+def _binary_erosion(input, structure, iterations, mask, output,
+                    border_value, origin, invert, brute_force):
+    input_rank = _ni_support._array_rank(input)
+    if structure == None:
+        structure = \
+            generate_binary_structure(input_rank, 1)
+    structure = numarray.asarray(structure)
+    if structure.rank != input_rank:
+        raise RuntimeError, 'structure rank must equal input rank'
+    origin = _ni_support._normalize_sequence(origin, input)
+    cit = _center_is_true(structure, origin)
+    inplace = isinstance(output, numarray.NumArray)
+    if iterations == 1:
+        result = _nd_image.binary_erosion(input, structure, mask, output,
+                                          border_value, origin, invert,
+                                          cit, 0)
+        if inplace:
+            return None
+        else:
+            return result[0]
+    elif cit and not brute_force:
+        result = _nd_image.binary_erosion(input, structure, mask, output,
+                                          border_value, origin, invert,
+                                          cit, 1)
+        if inplace:
+            changed, coordinate_list = result
+        else:
+            output, changed, coordinate_list = result
+        structure = structure[tuple([slice(None, None, -1)] *
+                                    structure.rank)]
+        for ii in range(len(origin)):
+            origin[ii] = -origin[ii]
+            if not structure.shape[ii] & 1:
+                origin[ii] -= 1
+        if mask != None:
+            mask = numarray.asarray(mask)
+            if mask.type() == numarray.Int8:
+                msk = mask.copy()
+            else:
+                msk = mask.astype(numarray.Int8)
+        else:
+            msk = None
+        _nd_image.binary_erosion2(output, structure, msk, iterations - 1,
+                                  origin, invert, coordinate_list)
+                
+        if not inplace:
+            return output
+    else:
+        tmp_in = numarray.zeros(input.shape, numarray.Bool)
+        if inplace:
+            tmp_out = output
+        else:
+            tmp_out = numarray.zeros(input.shape, numarray.Bool)
+        if not iterations & 1:
+            tmp_in, tmp_out = tmp_out, tmp_in
+        changed = _nd_image.binary_erosion(input, structure, mask, tmp_out,
+                                           border_value, origin, invert,
+                                           cit, 0)
+        ii = 1
+        while (ii < iterations) or (iterations < 1) and changed:
+            tmp_in, tmp_out = tmp_out, tmp_in
+            changed = _nd_image.binary_erosion(tmp_in, structure, mask,
+                                               tmp_out, border_value,
+                                               origin, invert, cit, 0)
+            ii += 1
+        if not inplace:
+            return tmp_out
+
+
 def binary_erosion(input, structure = None, iterations = 1, mask = None,
-                   output = None, border_value = 0, origin = 0):
+                   output = None, border_value = 0, origin = 0,
+                   brute_force = False):
     """Multi-dimensional binary erosion with the given structure.
     
     An output array can optionally be provided. The origin parameter
@@ -95,47 +170,18 @@ def binary_erosion(input, structure = None, iterations = 1, mask = None,
     modified at each iteration.
 
     """
-    if structure == None:
-        structure = generate_binary_structure(_ni_support._array_rank(input),
-                                          1)
-    origin = _ni_support._normalize_sequence(origin, input)
-    inplace = isinstance(output, numarray.NumArray)
-    if iterations == 1:
-        result = _nd_image.binary_erosion(input, structure, mask, output,
-                                          border_value, origin, 0, 0)
-        if inplace:
-            return None
-        else:
-            return result[0]
-    else:
-        tmp_in = numarray.zeros(input.shape, numarray.Bool)
-        if inplace:
-            tmp_out = output
-        else:
-            tmp_out = numarray.zeros(input.shape, numarray.Bool)
-        if not iterations & 1:
-            tmp_in, tmp_out = tmp_out, tmp_in
-        changed = _nd_image.binary_erosion(input, structure, mask, tmp_out,
-                                           border_value, origin, 0, 0)
-        ii = 1
-        while (ii < iterations) or (iterations < 1) and changed:
-            tmp_in, tmp_out = tmp_out, tmp_in
-            changed = _nd_image.binary_erosion(tmp_in, structure, mask,
-                                               tmp_out, border_value,
-                                               origin, 0, 0)
-            ii += 1
-        if not inplace:
-            return tmp_out
-
+    return _binary_erosion(input, structure, iterations, mask,
+                           output, border_value, origin, 0, brute_force)
 
 def binary_dilation(input, structure = None, iterations = 1, mask = None,
-                    output = None, border_value = 0, origin = 0):
+                    output = None, border_value = 0, origin = 0,
+                    brute_force = False):
     """Multi-dimensional binary dilation with the given structure.
     
     An output array can optionally be provided. The origin parameter
     controls the placement of the filter. If no structuring element is
     provided an element is generated with a squared connectivity equal
-    to one. The erosion operation is repeated iterations times.  If
+    to one. The dilation operation is repeated iterations times.  If
     iterations is less than 1, the dilation is repeated until the
     result does not change anymore.  If a mask is given, only those
     elements with a true value at the corresponding mask element are
@@ -144,8 +190,8 @@ def binary_dilation(input, structure = None, iterations = 1, mask = None,
     """
 
     if structure == None:
-        structure = generate_binary_structure(_ni_support._array_rank(input),
-                                              1)
+        structure = \
+            generate_binary_structure(_ni_support._array_rank(input), 1)
     origin = _ni_support._normalize_sequence(origin, input)
     structure = numarray.asarray(structure)
     structure = structure[tuple([slice(None, None, -1)] *
@@ -154,35 +200,8 @@ def binary_dilation(input, structure = None, iterations = 1, mask = None,
         origin[ii] = -origin[ii]
         if not structure.shape[ii] & 1:
             origin[ii] -= 1
-    # implemented by eroding the background with the reflected structure:
-    inplace = isinstance(output, numarray.NumArray)
-    if iterations == 1:
-        result = _nd_image.binary_erosion(input, structure, mask, output,
-                                          border_value, origin, 1, 1)
-        if inplace:
-            return None
-        else:
-            return result[0]
-    else:
-        tmp_in = numarray.zeros(input.shape, numarray.Bool)
-        if inplace:
-            tmp_out = output
-        else:
-            tmp_out = numarray.zeros(input.shape, numarray.Bool)
-        if not iterations & 1:
-            tmp_in, tmp_out = tmp_out, tmp_in
-        changed = _nd_image.binary_erosion(input, structure, mask,
-                                           tmp_out, border_value,
-                                           origin, 1, 1)
-        ii = 1
-        while (ii < iterations) or (iterations < 1) and changed:
-            tmp_in, tmp_out = tmp_out, tmp_in
-            changed = _nd_image.binary_erosion(tmp_in, structure, mask,
-                                               tmp_out, border_value,
-                                               origin, 1, 1)
-            ii += 1
-        if not inplace:
-            return tmp_out
+    return _binary_erosion(input, structure, iterations, mask,
+                           output, border_value, origin, 1, brute_force)
 
 
 def binary_opening(input, structure = None, iterations = 1, output = None,
@@ -213,7 +232,7 @@ def binary_closing(input, structure = None, iterations = 1, output = None,
     controls the placement of the filter. If no structuring element is
     provided an element is generated with a squared connectivity equal
     to one. The iterations parameter gives the number of times the
-    erosions and then the dilations are done.
+    dilations and then the erosions are done.
 
     """
     if structure == None:
@@ -239,8 +258,8 @@ def binary_hit_or_miss(input, structure1 = None, structure2 = None,
 
     """
     if structure1 == None:
-        structure1 = generate_binary_structure(_ni_support._array_rank(input),
-                                               1)
+        structure1 = \
+            generate_binary_structure(_ni_support._array_rank(input), 1)
     if structure2 == None:
         structure2 = numarray.logical_not(structure1)
 
@@ -250,16 +269,56 @@ def binary_hit_or_miss(input, structure1 = None, structure2 = None,
     else:
         origin2 = _ni_support._normalize_sequence(origin2, input)
 
-    tmp1, changed = _nd_image.binary_erosion(input, structure1, None, None,
-                                             0, origin1, 0, 0)
+    tmp1 = _binary_erosion(input, structure1, 1, None, None, 0, origin1, 0,
+                           False)
     inplace = isinstance(output, numarray.NumArray)
-    result= _nd_image.binary_erosion(input, structure2, None, output,
-                                     0, origin2, 1, 0)
+    result = _binary_erosion(input, structure2, 1, None, output, 0,
+                             origin2, 1, False)
+    
     if inplace:
+        numarray.logical_not(output, output)
         numarray.logical_and(tmp1, output, output)
     else:
-        return numarray.logical_and(tmp1, result[0])
+        numarray.logical_not(result, result)
+        return numarray.logical_and(tmp1, result)
+        
+def binary_propagation(input, structure = None, mask = None,
+                       output = None, border_value = 0, origin = 0):
+    """Multi-dimensional binary propagation with the given structure.
+    
+    An output array can optionally be provided. The origin parameter
+    controls the placement of the filter. If no structuring element is
+    provided an element is generated with a squared connectivity equal
+    to one. If a mask is given, only those elements with a true value at
+    the corresponding mask element are.
 
+    This function is functionally equivalent to calling binary_dilation
+    with the number of iterations less then one: iterative dilation until
+    the result does not change anymore.
+    """
+
+    return binary_dilation(input, structure, -1, mask, output, border_value,
+                           origin)   
+                           
+def binary_fill_holes(input, structure = None, output = None, origin = 0):
+    """Fill the holes in binary objects.
+    
+    An output array can optionally be provided. The origin parameter
+    controls the placement of the filter. If no structuring element is
+    provided an element is generated with a squared connectivity equal
+    to one.
+    """
+    mask = numarray.logical_not(input)
+    tmp = numarray.zeros(mask.shape, numarray.Bool)
+    inplace = isinstance(output, numarray.NumArray)
+    if inplace:
+        binary_dilation(tmp, structure, -1, mask, output, 1, origin)
+        numarray.logical_not(output, output)
+    else:
+        output = binary_dilation(tmp, structure, -1, mask, None, 1, origin)
+        numarray.logical_not(output, output)
+        return output
+     
 
 def grey_erosion(input,  size = None, footprint = None, structure = None,
                  output = None, mode = "nearest", cval = 0.0, origin = 0):
