@@ -46,7 +46,7 @@ int NI_GetArrayRank(PyArrayObject *array)
   return array->nd;
 }
 
-/* return the rank of an array */
+/* return the type of an array */
 NumarrayType NI_GetArrayType(PyArrayObject *array)
 {
   assert(array != NULL);
@@ -132,7 +132,14 @@ int NI_OutputArray(NumarrayType type, int rank, int *dimensions,
     }
   } else {
     /* An object is provided, call NA_OutputArray: */
-    *(PyArrayObject**)output = NA_OutputArray((PyObject*)output_in, type, 
+    if (!NA_NumArrayCheck(output_in)) {
+      PyErr_SetString(PyExc_RuntimeError, "output must be an array");
+      return 0;
+    } else if (((PyArrayObject*)output_in)->descr->type_num != type) {
+      PyErr_SetString(PyExc_RuntimeError, "output type incorrect");
+      return 0;
+    }
+    *(PyArrayObject**)output = NA_OutputArray(output_in, type, 
                                               NUM_ALIGNED|NUM_NOTSWAPPED);
     if (!*output) {
       PyErr_SetString(PyExc_RuntimeError, "cannot convert output array");
@@ -655,18 +662,24 @@ exit:
   }
 }
 
+static void _FreeCoordinateList(void* ptr) 
+{
+  NI_FreeCoordinateList((NI_CoordinateList*)ptr);
+}
+
 static PyObject *Py_BinaryErosion(PyObject *obj, PyObject *args)
 {
   PyArrayObject *input = NULL, *output = NULL, *strct = NULL, *mask = NULL;
   PyObject *output_object = NULL, *shifts_object = NULL;
-  PyObject *mask_object = NULL;
-  int border_value, *shifts = NULL, nshifts, invert_input, invert_output;
-  int changed = 0;
+  PyObject *mask_object = NULL, *cobj = NULL;
+  int border_value, *shifts = NULL, nshifts, invert, center_is_true;
+  int changed = 0, return_coordinates;
+  NI_CoordinateList *coordinate_list = NULL;
 
-  if (PyArg_ParseTuple(args, "O&O&OOiOii", NI_ObjectToArray, &input, 
+  if (PyArg_ParseTuple(args, "O&O&OOiOiii", NI_ObjectToArray, &input, 
                        NI_ObjectToContiguousBool, &strct, &mask_object,
                        &output_object, &border_value, &shifts_object, 
-                       &invert_input, &invert_output)) {
+                       &invert, &center_is_true, &return_coordinates)) {
     
     if (NI_ObjectToInts(shifts_object, &shifts, &nshifts)) {
       if (mask_object != Py_None) {
@@ -678,12 +691,17 @@ static PyObject *Py_BinaryErosion(PyObject *obj, PyObject *args)
       }
       if (!PyErr_Occurred()) {
         NI_BinaryErosion(input, strct, mask, &output, output_object, 
-                         border_value, shifts, invert_input,
-                         invert_output, &changed);
+                         border_value, shifts, invert, center_is_true,
+                         &changed, 
+                         return_coordinates ? &coordinate_list : NULL);
       }
     }
   }
 
+  if (return_coordinates) {
+    cobj = PyCObject_FromVoidPtr(coordinate_list, _FreeCoordinateList);
+  }
+
   Py_XDECREF(input);
   Py_XDECREF(strct);
   Py_XDECREF(mask);
@@ -691,17 +709,70 @@ static PyObject *Py_BinaryErosion(PyObject *obj, PyObject *args)
     free(shifts);
   if (PyErr_Occurred()) {
     Py_XDECREF(output);
+    Py_XDECREF(cobj);
     return NULL;
   } else {
     if (!output_object || output_object == Py_None) {
-      return Py_BuildValue("Ni", (PyObject*)output, changed);
+      if (return_coordinates) {
+        return Py_BuildValue("NiN", (PyObject*)output, changed, cobj);
+      } else {
+        return Py_BuildValue("Ni", (PyObject*)output, changed);
+      }
     } else {
       Py_XDECREF(output);
-      return Py_BuildValue("i", changed);
+      if (return_coordinates) {
+        return Py_BuildValue("iN", changed, cobj);
+      } else {
+        return Py_BuildValue("i", changed);
+      }
     }
   }
 }
 
+static PyObject *Py_BinaryErosion2(PyObject *obj, PyObject *args)
+{
+  PyArrayObject *array = NULL, *strct = NULL, *mask = NULL;
+  PyObject *mask_object = NULL, *shifts_object = NULL, *cobj = NULL;
+  int *shifts = NULL, nshifts, invert, niter;
+
+  if (PyArg_ParseTuple(args, "O&O&OiOiO", NI_ObjectToIoArray, &array, 
+                       NI_ObjectToContiguousBool, &strct, &mask_object, 
+                       &niter, &shifts_object, &invert, &cobj)) {
+    
+    if (NI_ObjectToInts(shifts_object, &shifts, &nshifts)) {
+      if (mask_object != Py_None) {
+        mask = NA_InputArray(mask_object, tAny,
+                             NUM_ALIGNED|NUM_NOTSWAPPED);
+        if (!mask) {
+          PyErr_SetString(PyExc_RuntimeError, "cannot convert mask");
+        }
+      }
+      if (!PyErr_Occurred()) {
+        if (PyCObject_Check(cobj)) {
+          NI_CoordinateList *cobj_data = PyCObject_AsVoidPtr(cobj);
+          NI_BinaryErosion2(array, strct, mask, niter, shifts, invert,
+                            &cobj_data);
+        } else {
+          PyErr_SetString(PyExc_RuntimeError, "cannot convert CObject");
+        }
+      }
+    }
+  }
+
+  Py_XDECREF(array);
+  Py_XDECREF(strct);
+  Py_XDECREF(mask);
+  if (shifts)
+    free(shifts);
+  if (PyErr_Occurred()) {
+    return NULL;
+  } else {
+    Py_INCREF(Py_None);
+    return Py_None;
+  }
+}
+
+
 static PyObject *Py_DistanceTransformOnePass(PyObject *obj, PyObject *args)
 {
   PyArrayObject *strct = NULL, *distances = NULL, *features = NULL;
@@ -1534,6 +1605,27 @@ int _NI_GetLabels(PyObject* labels_object, PyArrayObject** labels)
   return 1;
 }
 
+PyObject* _NI_BuildMeasurementResultArrayObject(int n_results,
+                                                PyArrayObject** values)
+{
+  PyObject *result = NULL;
+  if (n_results > 1) {
+    result = PyList_New(n_results);
+    if (result) {
+      int ii;
+      for(ii = 0; ii < n_results; ii++) {
+        PyList_SET_ITEM(result, ii, (PyObject*)values[ii]);
+        Py_XINCREF(values[ii]);
+      }
+    }
+  } else {
+    result = (PyObject*)values[0];
+    Py_XINCREF(values[0]);
+  }
+  return result;
+}
+
+
 PyObject* _NI_BuildMeasurementResultDouble(int n_results, double* values)
 {
   PyObject *result = NULL;
@@ -1935,6 +2027,71 @@ static PyObject *Py_MaximumPosition(PyObject *obj, PyObject *args)
 }
 
 
+static PyObject *Py_Extrema(PyObject *obj, PyObject *args)
+{
+  PyArrayObject *input = NULL, *labels = NULL;
+  PyObject *labels_object = NULL, *indices_object, *result = NULL;
+  PyObject *min_result = NULL, *minp_result = NULL;
+  PyObject *max_result = NULL, *maxp_result = NULL;
+  double *minimum = NULL, *maximum = NULL;
+  int *minimum_position = NULL, *maximum_position = NULL;
+  int min_label, max_label, *result_indices = NULL, n_results;
+  
+  if (!PyArg_ParseTuple(args, "O&OO", NI_ObjectToArray, &input, 
+                        &labels_object, &indices_object))
+    goto exit;
+
+  if (!_NI_GetIndices(indices_object, &result_indices, &min_label,
+                      &max_label, &n_results))
+    goto exit;
+
+  if (!_NI_GetLabels(labels_object, &labels))
+    goto exit;
+
+  minimum = (double*)malloc(n_results * sizeof(double));
+  maximum = (double*)malloc(n_results * sizeof(double));
+  minimum_position = (int*)malloc(n_results * sizeof(int));
+  maximum_position = (int*)malloc(n_results * sizeof(int));
+  if (!minimum || !minimum_position || !maximum || !maximum_position) {
+    PyErr_NoMemory();
+    goto exit;
+  }
+  
+  if (!NI_Statistics(input, labels, min_label, max_label, result_indices, 
+                     n_results, NULL, NULL, NULL, minimum, maximum, 
+                     minimum_position, maximum_position))
+    goto exit;
+
+  min_result = _NI_BuildMeasurementResultDouble(n_results, minimum);
+  max_result = _NI_BuildMeasurementResultDouble(n_results, maximum);
+  minp_result = _NI_BuildMeasurementResultInt(n_results, minimum_position);
+  maxp_result = _NI_BuildMeasurementResultInt(n_results, maximum_position);
+  if (!min_result || !minp_result || !max_result || !maxp_result)
+    goto exit;
+    
+  result = Py_BuildValue("OOOO", min_result, max_result, minp_result, 
+                         maxp_result);
+
+ exit:
+  Py_XDECREF(input);
+  Py_XDECREF(labels);
+  Py_XDECREF(min_result);
+  Py_XDECREF(minp_result);
+  Py_XDECREF(max_result);
+  Py_XDECREF(maxp_result);
+  if (result_indices)
+    free(result_indices);
+  if (minimum_position)
+    free(minimum_position);
+  if (minimum)
+    free(minimum);
+  if (maximum_position)
+    free(maximum_position);
+  if (maximum)
+    free(maximum);
+  return result;
+}
+
 static PyObject *Py_CenterOfMass(PyObject *obj, PyObject *args)
 {
   PyArrayObject *input = NULL, *labels = NULL;
@@ -1976,6 +2133,52 @@ static PyObject *Py_CenterOfMass(PyObject *obj, PyObject *args)
   return result;
 }
 
+static PyObject *Py_Histogram(PyObject *obj, PyObject *args)
+{
+  PyArrayObject *input = NULL, *labels = NULL, **histograms = NULL;
+  PyObject *labels_object = NULL, *indices_object, *result = NULL;
+  int min_label, max_label, *result_indices = NULL, n_results, jj, nbins;
+  double min, max;
+  
+  if (!PyArg_ParseTuple(args, "O&ddiOO", NI_ObjectToArray, &input, 
+                        &min, &max, &nbins, &labels_object,
+                        &indices_object))
+    goto exit;
+
+  if (!_NI_GetIndices(indices_object, &result_indices, &min_label,
+                      &max_label, &n_results))
+    goto exit;
+
+  if (!_NI_GetLabels(labels_object, &labels))
+    goto exit;
+
+  histograms = (PyArrayObject**)malloc(input->nd * n_results *
+                                       sizeof(PyArrayObject*));
+  if (!histograms) {
+    PyErr_NoMemory();
+    goto exit;
+  }
+  
+  if (!NI_Histogram(input, labels, min_label, max_label, result_indices, 
+                    n_results, histograms, min, max, nbins))
+    goto exit;
+
+  result = _NI_BuildMeasurementResultArrayObject(n_results, histograms);
+
+ exit:
+  Py_XDECREF(input);
+  Py_XDECREF(labels);
+  if (result_indices)
+    free(result_indices);
+  if (histograms) {
+    for(jj = 0; jj < n_results; jj++) {
+      Py_XDECREF(histograms[jj]);
+    }
+    free(histograms);
+  }
+  return result;
+}
+
 
 static PyMethodDef methods[] = {
   {"correlate1d", (PyCFunction)Py_Correlate1D, METH_VARARGS, ""},
@@ -1989,6 +2192,7 @@ static PyMethodDef methods[] = {
   {"generic_filter", (PyCFunction)Py_GenericFilter, METH_VARARGS, ""},
   {"generic_filter1d", (PyCFunction)Py_GenericFilter1D, METH_VARARGS, ""},
   {"binary_erosion", (PyCFunction)Py_BinaryErosion, METH_VARARGS, ""},
+  {"binary_erosion2", (PyCFunction)Py_BinaryErosion2, METH_VARARGS, ""},
   {"distance_transform_bf", (PyCFunction)Py_DistanceTransformBruteForce, 
    METH_VARARGS, ""},
   {"distance_transform_op", (PyCFunction)Py_DistanceTransformOnePass, 
@@ -2012,7 +2216,9 @@ static PyMethodDef methods[] = {
   {"maximum", (PyCFunction)Py_Maximum, METH_VARARGS, ""},
   {"minimum_position", (PyCFunction)Py_MinimumPosition, METH_VARARGS, ""},
   {"maximum_position", (PyCFunction)Py_MaximumPosition, METH_VARARGS, ""},
+  {"extrema", (PyCFunction)Py_Extrema, METH_VARARGS, ""},
   {"center_of_mass", (PyCFunction)Py_CenterOfMass, METH_VARARGS, ""},
+  {"histogram", (PyCFunction)Py_Histogram, METH_VARARGS, ""},
   {"fourier_boxcar", (PyCFunction)Py_FourierBoxcar, METH_VARARGS, ""},
   {"fourier_ellipsoid", (PyCFunction)Py_FourierEllipsoid, METH_VARARGS,
    ""},
