@@ -284,7 +284,7 @@ int NI_Correlate(PyArrayObject* input, PyArrayObject* weights,
 
   /* calculate the filter offsets: */
   if (!NI_InitFilterOffsetsFromArray(input, footprint, shifts, mode,
-                                     &offsets, &border_flag_value))
+                                     &offsets, &border_flag_value, NULL))
     goto exit;
 
   /* initialize input element iterator: */
@@ -759,7 +759,7 @@ int NI_MinimumMaximumFilter(PyArrayObject* input, PyArrayObject* footprint,
 
   /* calculate the filter offsets: */
   if (!NI_InitFilterOffsetsFromArray(input, footprint, shifts, mode,
-                                     &offsets, &border_flag_value))
+                                     &offsets, &border_flag_value, NULL))
     goto exit;
 
   /* initialize input element iterator: */
@@ -978,7 +978,7 @@ int NI_RankFilter(PyArrayObject* input, int filter_rank,
 
   /* calculate the filter offsets: */
   if (!NI_InitFilterOffsetsFromArray(input, footprint, shifts, mode,
-                                     &offsets, &border_flag_value))
+                                     &offsets, &border_flag_value, NULL))
     goto exit;
 
   /* initialize input element iterator: */
@@ -1155,7 +1155,7 @@ int NI_GenericFilter(PyArrayObject* input,
 
   /* calculate the filter offsets: */
   if (!NI_InitFilterOffsetsFromArray(input, footprint, shifts, mode,
-                                     &offsets, &border_flag_value))
+                                     &offsets, &border_flag_value, NULL))
     goto exit;
 
   /* initialize input element iterator: */
