@@ -171,7 +171,7 @@ int NI_Label(PyArrayObject* input, PyArrayObject* strct, int *max_label,
   /* calculate the filter offsets: */
   if (!NI_InitFilterOffsetsFromArray(*output, tmp, shifts,
                                      NI_EXTEND_CONSTANT, &offsets,
-                                     &mask_value))
+                                     &mask_value, NULL))
     goto exit;
   /* initialize filter iterator: */
   if (!NI_InitFilterIterator(irank, sdims, filter_size,  idims, shifts,
@@ -737,7 +737,8 @@ int NI_Statistics(PyArrayObject *input, PyArrayObject *labels,
         }
       }
       for(jj = 0; jj < n_results; jj++)
-        variance[jj] = total[jj] > 1 ? variance[jj] / (total[jj] - 1) : 0.0;
+        variance[jj] = (total[jj] > 1 ? 
+                        variance[jj] / (total[jj] - 1) : 0.0);
     }
   }
   
@@ -746,8 +747,8 @@ int NI_Statistics(PyArrayObject *input, PyArrayObject *labels,
 
 
 int NI_CenterOfMass(PyArrayObject *input, PyArrayObject *labels, 
-                    int min_label, int max_label, int *indices, int n_results, 
-                    double *center_of_mass)
+                    int min_label, int max_label, int *indices, 
+                    int n_results, double *center_of_mass)
 {
   char *pi = NULL, *pm = NULL;
   NI_Iterator ii, mi;
@@ -831,6 +832,107 @@ int NI_CenterOfMass(PyArrayObject *input, PyArrayObject *labels,
 }
 
 
+int NI_Histogram(PyArrayObject *input, PyArrayObject *labels, 
+                 int min_label, int max_label, int *indices, 
+                 int n_results, PyArrayObject **histograms,
+                 double min, double max, int nbins)
+{
+  char *pi = NULL, *pm = NULL;
+  NI_Iterator ii, mi;
+  int jj, kk, size, mtype = 0, itype = 0, idx = 0, label = 1, doit = 1;
+  Int32 **ph = NULL;
+  double bsize;
+  
+  assert(input != NULL);
+  
+  if (nbins < 1) {
+    PyErr_SetString(PyExc_RuntimeError,
+                    "number of bins must be at least one");
+    goto exit;
+  }
+ 
+  if (min >= max) {
+    PyErr_SetString(PyExc_RuntimeError,
+                    "histogram maximum must be larger than its minimum");
+    goto exit;
+  }
+ 
+  /* input iterator: */
+  if (!NI_InitPointIterator(input, &ii))
+    goto exit;
+  /* input data: */
+  pi = NI_GetArrayData(input);
+  /* input type: */
+  itype = NI_GetArrayType(input);
+  if (labels) {
+    /* input and labels must have equal size: */
+    if (!NI_ShapeEqual(input, labels)) {
+      PyErr_SetString(PyExc_RuntimeError, 
+                      "input and labels dimensions must be equal");
+      goto exit;
+    }
+    /* iterator, data pointer and type of labels array: */
+    if (!NI_InitPointIterator(labels, &mi))
+      goto exit;
+    pm = NI_GetArrayData(labels);
+    mtype = NI_GetArrayType(labels);
+  }
+
+  ph = (Int32**)malloc(n_results * sizeof(Int32*));
+  if (!ph) {
+    PyErr_NoMemory();
+    goto exit;
+  }
+  for(jj = 0; jj < n_results; jj++) {
+    histograms[jj] = NI_NewArray(tInt32, 1, &nbins);
+    if (!histograms[jj]) {
+      PyErr_NoMemory();
+      goto exit;
+    }
+    ph[jj] = (Int32*)NI_GetArrayData(histograms[jj]);
+    for(kk = 0; kk < nbins; kk++)
+      ph[jj][kk] = 0;
+  }
+
+  bsize = (max - min) / (double)nbins;
+
+  /* input size: */
+  size = NI_Elements(input);
+  /* iterate over array: */
+  for(jj = 0; jj < size; jj++) {
+    NI_GET_LABEL(pm, label, mtype);
+    if (min_label >= 0) {
+      if (label >= min_label && label <= max_label) {
+        idx = indices[label - min_label];
+        doit = idx >= 0;
+      } else {
+        doit = 0;
+      } 
+    } else {
+      doit = label != 0;
+    }
+    if (doit) {
+      int bin;
+      double val;
+      NI_GET_VALUE(pi, val, itype);
+      if (val >= min && val < max) {
+        bin = (int)((val - min) / bsize);
+        ++(ph[idx][bin]);
+      }
+    }
+    if (labels) {
+      NI_ITERATOR_NEXT2(ii, mi, pi, pm);
+    } else {
+      NI_ITERATOR_NEXT(ii, pi);
+    }
+  }
+
+ exit:
+  if (ph)
+    free(ph);
+  return  PyErr_Occurred() == NULL;
+}
+
 #define DONE_TYPE UInt8
 #define COST_TYPE UInt16
 #define WS_MAXDIM 7
