@@ -37,48 +37,56 @@
 #include <limits.h>
 #include <float.h>
 
-#define NI_ERODE_POINT(_pi, _out, _offsets, _filter_size, _type, _mv,     \
-                       _border_value, _bv, _true, _false, _ivo, _changed) \
-{                                                                         \
-  int _ii, _oo = *_offsets;                                               \
-  int _in = *(_type*)_pi ? 1 : 0;                                         \
-  if (_mv) {                                                              \
-    _out = 1;                                                             \
-    for(_ii = 0; _ii < _filter_size; _ii++) {                             \
-      _oo = _offsets[_ii];                                                \
-      if (_oo == _bv) {                                                   \
-        if (!_border_value) {                                             \
-          _out = 0;                                                       \
-          break;                                                          \
-        }                                                                 \
-      } else {                                                            \
-        int _nn = *(_type*)(_pi + _oo) ? _true : _false;                  \
-        if (!_nn) {                                                       \
-          _out = 0;                                                       \
-          break;                                                          \
-        }                                                                 \
-      }                                                                   \
-    }                                                                     \
-    _out = _ivo ? 1 - _out : _out;                                        \
-    if (_in != _out)                                                      \
-      _changed = 1;                                                       \
-  } else {                                                                \
-    _out = _in;                                                           \
-  }                                                                       \
+#define LIST_SIZE 100000
+
+#define NI_ERODE_POINT(_pi, _out, _offsets, _filter_size, _type, _mv, \
+                       _border_value, _bv, _center_is_true, _true,    \
+                       _false, _changed)                              \
+{                                                                     \
+  int _ii, _oo;                                                       \
+  int _in = *(_type*)_pi ? 1 : 0;                                     \
+  if (_mv) {                                                          \
+    if (_center_is_true && _in == false) {                            \
+      _changed = 0;                                                   \
+      _out = _in;                                                     \
+    } else {                                                          \
+      _out = _true;                                                   \
+      for(_ii = 0; _ii < _filter_size; _ii++) {                       \
+        _oo = _offsets[_ii];                                          \
+        if (_oo == _bv) {                                             \
+          if (!_border_value) {                                       \
+            _out = _false;                                            \
+            break;                                                    \
+          }                                                           \
+        } else {                                                      \
+          int _nn = *(_type*)(_pi + _oo) ? _true : _false;            \
+          if (!_nn) {                                                 \
+            _out = _false;                                            \
+            break;                                                    \
+          }                                                           \
+        }                                                             \
+      }                                                               \
+      _changed = _out != _in;                                         \
+    }                                                                 \
+  } else {                                                            \
+    _out = _in;                                                       \
+  }                                                                   \
 }                                                                         
 
 int NI_BinaryErosion(PyArrayObject* input, PyArrayObject* strct, 
                      PyArrayObject* mask, PyArrayObject** output,  
                      PyObject* output_in, int bdr_value, int *shifts, 
-                     int invert_input, int invert_output,  int* changed)
+                     int invert, int center_is_true, int* changed, 
+                     NI_CoordinateList **coordinate_list)
 {
   int struct_size = 0, *offsets = NULL, size, *oo, jj, border_flag_value;
   int ssize, mtype = tAny, true, false, msk_value, irank, itype;
-  int idims[NI_MAXDIM], sdims[NI_MAXDIM];
+  int idims[NI_MAXDIM], sdims[NI_MAXDIM], block_size = 0, *current = NULL;
   NI_Iterator ii, io, mi;
   NI_FilterIterator fi;
   Bool *ps, out = 0;
   char *pi, *po, *pm = NULL;
+  NI_CoordinateBlock *block = NULL;
 
   assert(input != NULL);
   assert(output != NULL);
@@ -144,7 +152,7 @@ int NI_BinaryErosion(PyArrayObject* input, PyArrayObject* strct,
   /* calculate the filter offsets: */
   if (!NI_InitFilterOffsetsFromArray(input, strct, shifts,
                                      NI_EXTEND_CONSTANT, &offsets,
-                                     &border_flag_value))
+                                     &border_flag_value, NULL))
     goto exit;
 
   /* initialize input element iterator: */
@@ -157,7 +165,8 @@ int NI_BinaryErosion(PyArrayObject* input, PyArrayObject* strct,
 
   /* initialize filter iterator: */
   NI_GetArrayDimensions(strct, sdims);
-  if (!NI_InitFilterIterator(irank, sdims, struct_size, idims, shifts, &fi))
+  if (!NI_InitFilterIterator(irank, sdims, struct_size, idims, shifts,
+                             &fi))
     goto exit;
     
   /* get data pointers an size: */
@@ -165,7 +174,7 @@ int NI_BinaryErosion(PyArrayObject* input, PyArrayObject* strct,
   po = NI_GetArrayData(*output);
   size = NI_Elements(input);
 
-  if (invert_input) {
+  if (invert) {
     bdr_value = bdr_value ? 0 : 1;
     true = 0;
     false = 1;
@@ -174,12 +183,24 @@ int NI_BinaryErosion(PyArrayObject* input, PyArrayObject* strct,
     true = 1;
     false = 0;
   }
+  
+  if (coordinate_list) {
+    block_size = LIST_SIZE / irank / sizeof(int);
+    if (block_size < 1)
+      block_size = 1;
+    if (block_size > size)
+      block_size = size;
+    *coordinate_list = NI_InitCoordinateList(block_size, irank);
+    if (!*coordinate_list)
+      goto exit;
+  }
 
   /* iterator over the elements: */
   oo = offsets;
   *changed = 0;
   msk_value = 1;
   for(jj = 0; jj < size; jj++) {
+    int pchange = 0, kk;
     if (mask) {                                                        
       switch(mtype) {                                                
       case tBool:                        
@@ -224,67 +245,79 @@ int NI_BinaryErosion(PyArrayObject* input, PyArrayObject* strct,
     }
     switch (itype) {
     case tBool:
-      NI_ERODE_POINT(pi, out, oo, struct_size, Bool, msk_value, bdr_value, 
-                     border_flag_value, true, false, invert_output,
-                     *changed);
+      NI_ERODE_POINT(pi, out, oo, struct_size, Bool, msk_value,
+                     bdr_value, border_flag_value, center_is_true,
+                     true, false, pchange);
       break;
     case tUInt8:
-      NI_ERODE_POINT(pi, out, oo, struct_size, UInt8, msk_value, bdr_value,
-                     border_flag_value, true, false, invert_output,
-                     *changed);
+      NI_ERODE_POINT(pi, out, oo, struct_size, UInt8, msk_value,
+                     bdr_value, border_flag_value, center_is_true,
+                     true, false, pchange);
       break;
     case tUInt16:
-      NI_ERODE_POINT(pi, out, oo, struct_size, UInt16, msk_value, bdr_value,
-                     border_flag_value, true, false, invert_output,
-                     *changed);
+      NI_ERODE_POINT(pi, out, oo, struct_size, UInt16, msk_value, 
+                     bdr_value, border_flag_value, center_is_true, 
+                     true, false, pchange);
       break;
     case tUInt32:
-      NI_ERODE_POINT(pi, out, oo, struct_size, UInt32, msk_value, bdr_value,
-                     border_flag_value, true, false, invert_output,
-                     *changed);
+      NI_ERODE_POINT(pi, out, oo, struct_size, UInt32, msk_value,
+                     bdr_value, border_flag_value, center_is_true,
+                     true, false, pchange);
       break;
 #if HAS_UINT64
     case tUInt64:
-      NI_ERODE_POINT(pi, out, oo, struct_size, UInt64, msk_value, bdr_value,
-                     border_flag_value, true, false, invert_output,
-                     *changed);
+      NI_ERODE_POINT(pi, out, oo, struct_size, UInt64, msk_value, 
+                     bdr_value, border_flag_value, center_is_true,
+                     true, false, pchange);
       break;
 #endif
     case tInt8:
-      NI_ERODE_POINT(pi, out, oo, struct_size, Int8, msk_value, bdr_value, 
-                     border_flag_value, true, false, invert_output,
-                     *changed);
+      NI_ERODE_POINT(pi, out, oo, struct_size, Int8, msk_value,
+                     bdr_value, border_flag_value, center_is_true,
+                     true, false, pchange);
       break;
     case tInt16:
-      NI_ERODE_POINT(pi, out, oo, struct_size, Int16, msk_value, bdr_value,
-                     border_flag_value, true, false, invert_output,
-                     *changed);
+      NI_ERODE_POINT(pi, out, oo, struct_size, Int16, msk_value,
+                     bdr_value, border_flag_value, center_is_true,
+                     true, false, pchange);
       break;
     case tInt32:
-      NI_ERODE_POINT(pi, out, oo, struct_size, Int32, msk_value, bdr_value, 
-                     border_flag_value, true, false, invert_output,
-                     *changed);
+      NI_ERODE_POINT(pi, out, oo, struct_size, Int32, msk_value,
+                     bdr_value, border_flag_value, center_is_true,
+                     true, false, pchange);
       break;
     case tInt64:
-      NI_ERODE_POINT(pi, out, oo, struct_size, Int64, msk_value, bdr_value, 
-                     border_flag_value, true, false, invert_output,
-                     *changed);
+      NI_ERODE_POINT(pi, out, oo, struct_size, Int64, msk_value,
+                     bdr_value, border_flag_value, center_is_true,
+                     true, false, pchange);
       break;
     case tFloat32:
       NI_ERODE_POINT(pi, out, oo, struct_size, Float32, msk_value, 
-                     bdr_value, border_flag_value, true, false,
-                    invert_output, *changed);
+                     bdr_value, border_flag_value, center_is_true,
+                     true, false, pchange);
       break;
     case tFloat64:
       NI_ERODE_POINT(pi, out, oo, struct_size, Float64, msk_value,
-                     bdr_value, border_flag_value, true, false,
-                     invert_output, *changed);
+                     bdr_value, border_flag_value, center_is_true,
+                     true, false, pchange);
       break;
     default:
       PyErr_SetString(PyExc_RuntimeError, "data type not supported");
       goto exit;
     }
     *(Bool*)po = out;
+    if (pchange) {
+      *changed = 1;
+      if (coordinate_list) {
+        if (block == NULL ||  block->size == block_size) {
+          block = NI_CoordinateListAddBlock(*coordinate_list);
+          current = block->coordinates;
+        }
+        for(kk = 0; kk < irank; kk++)
+          *current++ = ii.coordinates[kk];
+        block->size++;
+      }
+    }
     if (mask) {
       NI_FILTER_NEXT3(fi, ii, io, mi, oo, pi, po, pm);
     } else {
@@ -295,6 +328,301 @@ int NI_BinaryErosion(PyArrayObject* input, PyArrayObject* strct,
  exit:
   if (offsets) 
     free(offsets);
+  if (PyErr_Occurred()) {
+    if (coordinate_list) {
+      NI_FreeCoordinateList(*coordinate_list);
+      *coordinate_list = NULL;
+    }
+    return 0;
+  } else {
+    return 1;
+  }
+  return PyErr_Occurred() ? 0 : 1;
+}
+
+#define ERODE_POINT2(_struct_size, _offsets, _coordinate_offsets, _pi, \
+                     _oo, _irank, _list1, _list2, _current_coors1,     \
+                     _current_coors2, _block1, _block2, _bf_value,     \
+                     _true, _false, _type, _mklist)                    \
+{                                                                      \
+  int _hh, _kk;                                                        \
+  for(_hh = 0; _hh < _struct_size; _hh++) {                            \
+    int _to = _offsets[_oo + _hh];                                     \
+    if (_to != _bf_value && *(_type*)(_pi + _to) == _true) {           \
+      if (_mklist) {                                                   \
+        int *_tc = &(_coordinate_offsets[(_oo + _hh) * _irank]);       \
+        if (_block2 == NULL ||  _block2->size == _list2->block_size) { \
+          _block2 = NI_CoordinateListAddBlock(_list2);                 \
+          _current_coors2 = _block2->coordinates;                      \
+        }                                                              \
+        for(_kk = 0; _kk < _irank; _kk++)                              \
+          *_current_coors2++ = _current_coors1[_kk] + _tc[_kk];        \
+        _block2->size++;                                               \
+      }                                                                \
+      *(_type*)(_pi + _to) = _false;                                   \
+    }                                                                  \
+  }                                                                    \
+}
+
+int NI_BinaryErosion2(PyArrayObject* array, PyArrayObject* strct,
+                      PyArrayObject* mask, int niter, int *shifts,
+                      int invert, NI_CoordinateList **iclist)
+{
+  int struct_size = 0, *offsets = NULL, oo, jj, border_flag_value;
+  int ssize, true, false, irank, itype, *coordinate_offsets = NULL;
+  int idims[NI_MAXDIM], sdims[NI_MAXDIM], current = 0, size = 0;
+  int *current_coordinates1 = NULL, *current_coordinates2 = NULL;
+  NI_Iterator ii, mi;
+  NI_FilterIterator fi, ci;
+  Bool *ps;
+  char *pi, *ibase, *pm = NULL;
+  NI_CoordinateBlock *block1 = NULL, *block2 = NULL;
+  NI_CoordinateList *list1 = NULL, *list2 = NULL;
+  assert(array != NULL);
+  assert(strct != NULL);
+
+  /* complex type not supported: */
+  itype = NI_GetArrayType(array);
+  if (itype == tComplex32 || itype == tComplex64) {
+    PyErr_SetString(PyExc_RuntimeError, "complex arrays not supported");
+    goto exit;
+  }
+
+  irank = NI_GetArrayRank(array);
+  
+  /* check the rank of the coordinate list */
+  if ((*iclist)->rank != irank) {
+    PyErr_SetString(PyExc_RuntimeError, "coordinate list rank is wrong");
+    goto exit;
+  }
+
+  /* array and structure must have equal rank: */
+  if (irank != NI_GetArrayRank(strct)) {
+    PyErr_SetString(PyExc_RuntimeError, 
+                    "structure and input arrays must have equal rank");
+    goto exit;
+  }
+
+  /* structuring element must be of bool type: */
+  if (NI_GetArrayType(strct) != tBool) {
+    PyErr_SetString(PyExc_RuntimeError, "structure type must be boolean");
+    goto exit;
+  }
+
+  /* the structure array must be contigous: */
+  if (!PyArray_ISCONTIGUOUS(strct)) {
+    PyErr_SetString(PyExc_RuntimeError,
+                    "structure array must be contiguous");
+    goto exit;
+  }
+  ps = (Bool*)NI_GetArrayData(strct);
+  ssize = NI_Elements(strct);
+  for(jj = 0; jj < ssize; jj++) 
+    if (ps[jj]) ++struct_size;
+
+  /* structure must be not be emtpy: */
+  if (struct_size < 1) {
+    PyErr_SetString(PyExc_RuntimeError, "structure size must be > 0");
+    goto exit;
+  }
+
+  /* calculate the filter offsets: */
+  if (!NI_InitFilterOffsetsFromArray(array, strct, shifts,
+                                     NI_EXTEND_CONSTANT, &offsets,
+                                     &border_flag_value,
+                                     &coordinate_offsets))
+    goto exit;
+      
+  /* initialize input element iterator: */
+  if (!NI_InitPointIterator(array, &ii))
+    goto exit;
+
+  /* initialize filter iterator: */
+  NI_GetArrayDimensions(strct, sdims);
+  NI_GetArrayDimensions(array, idims);
+  if (!NI_InitFilterIterator(irank, sdims, struct_size, idims, shifts,
+                             &fi))
+    goto exit;
+  if (!NI_InitFilterIterator(irank, sdims, struct_size * irank, idims,
+                             shifts, &ci))
+    goto exit;
+    
+  /* get data pointers and size: */
+  ibase = pi = NI_GetArrayData(array);
+
+  if (invert) {
+    true = 0;
+    false = 1;
+  } else {
+    true = 1;
+    false = 0;
+  }
+
+  if (mask) {
+    /* input and mask must have equal size: */
+    if (!NI_ShapeEqual(array, mask)) {
+      PyErr_SetString(PyExc_RuntimeError, 
+                      "input and mask sizes must be equal");
+      return 0;
+    }
+    /* iterator, data pointer and type of mask array: */
+    if (!NI_InitPointIterator(mask, &mi))
+      return 0;
+    pm = NI_GetArrayData(mask);
+
+    size = NI_Elements(array);
+
+    for(jj = 0; jj < size; jj++) {
+      if (*(Int8*)pm) {
+        *(Int8*)pm = -1;
+      } else {
+        *(Int8*)pm = (Int8)*(Bool*)pi;
+        *(Bool*)pi = false;
+      }
+      NI_ITERATOR_NEXT2(ii, mi,  pi, pm)    
+    }
+    NI_ITERATOR_RESET(ii)
+    pi = NI_GetArrayData(array);
+  }
+
+  list1 = NI_InitCoordinateList((*iclist)->block_size, (*iclist)->rank);
+  list2 = NI_InitCoordinateList((*iclist)->block_size, (*iclist)->rank);
+  if (!list1 || !list2)
+    goto exit;
+  if (NI_CoordinateListStealBlocks(list2, *iclist))
+    goto exit;
+  block2 = list2->blocks;
+  jj = 0;
+  while(block1 || block2) {
+    int mklist = 1;
+    if (!block1) {
+      if (niter <= 0 || jj < niter) {
+        if (NI_CoordinateListStealBlocks(list1, list2))
+          goto exit;
+        block1 = list1->blocks;
+        block2 = NULL;
+        current_coordinates1 = block1->coordinates;
+        current = 0;
+        ++jj;      
+        mklist = niter <= 0 || jj < niter;   
+      } else {
+        break;
+      }      
+    }
+    NI_ITERATOR_GOTO(ii, current_coordinates1, ibase, pi);
+    NI_FILTER_GOTO(fi, ii, 0, oo);
+
+    switch (itype) {
+    case tBool:
+      ERODE_POINT2(struct_size, offsets, coordinate_offsets, pi,
+                   oo, irank, list1, list2, current_coordinates1, 
+                   current_coordinates2, block1, block2, 
+                   border_flag_value, true, false, Bool, mklist);
+      break;
+    case tUInt8:
+      ERODE_POINT2(struct_size, offsets, coordinate_offsets, pi,
+                   oo, irank, list1, list2, current_coordinates1, 
+                   current_coordinates2, block1, block2, 
+                   border_flag_value, true, false, UInt8, mklist);
+      break;
+    case tUInt16:
+      ERODE_POINT2(struct_size, offsets, coordinate_offsets, pi,
+                   oo, irank, list1, list2, current_coordinates1, 
+                   current_coordinates2, block1, block2, 
+                   border_flag_value, true, false, UInt16, mklist);
+      break;
+    case tUInt32:
+      ERODE_POINT2(struct_size, offsets, coordinate_offsets, pi,
+                   oo, irank, list1, list2, current_coordinates1, 
+                   current_coordinates2, block1, block2, 
+                   border_flag_value, true, false, UInt32, mklist);
+      break;
+#if HAS_UINT64
+    case tUInt64:
+      ERODE_POINT2(struct_size, offsets, coordinate_offsets, pi,
+                   oo, irank, list1, list2, current_coordinates1, 
+                   current_coordinates2, block1, block2, 
+                   border_flag_value, true, false, UInt64, mklist);
+      break;
+#endif
+    case tInt8:
+      ERODE_POINT2(struct_size, offsets, coordinate_offsets, pi,
+                   oo, irank, list1, list2, current_coordinates1, 
+                   current_coordinates2, block1, block2, 
+                   border_flag_value, true, false, Int8, mklist);
+      break;
+    case tInt16:
+      ERODE_POINT2(struct_size, offsets, coordinate_offsets, pi,
+                   oo, irank, list1, list2, current_coordinates1, 
+                   current_coordinates2, block1, block2, 
+                   border_flag_value, true, false, Int16, mklist);
+      break;
+    case tInt32:
+      ERODE_POINT2(struct_size, offsets, coordinate_offsets, pi,
+                   oo, irank, list1, list2, current_coordinates1, 
+                   current_coordinates2, block1, block2, 
+                   border_flag_value, true, false, Int32, mklist);
+      break;
+    case tInt64:
+      ERODE_POINT2(struct_size, offsets, coordinate_offsets, pi,
+                   oo, irank, list1, list2, current_coordinates1, 
+                   current_coordinates2, block1, block2, 
+                   border_flag_value, true, false, Int64, mklist);
+      break;
+    case tFloat32:
+      ERODE_POINT2(struct_size, offsets, coordinate_offsets, pi,
+                   oo, irank, list1, list2, current_coordinates1, 
+                   current_coordinates2, block1, block2, 
+                   border_flag_value, true, false, Float32, mklist);
+      break;
+    case tFloat64:  
+      ERODE_POINT2(struct_size, offsets, coordinate_offsets, pi,
+                   oo, irank, list1, list2, current_coordinates1, 
+                   current_coordinates2, block1, block2, 
+                   border_flag_value, true, false, Float64, mklist);
+      break;
+    default:
+      PyErr_SetString(PyExc_RuntimeError, "data type not supported");
+      goto exit;
+    }
+
+    ++current;
+    if (current == block1->size) {
+      block1 = NI_CoordinateListDeleteBlock(list1);
+      if (block1) {
+        current_coordinates1 = block1->coordinates;
+        current = 0;
+      }
+    } else {
+      current_coordinates1 += irank;
+    }
+  }
+  
+  if (mask) {
+    NI_ITERATOR_RESET(ii)
+    NI_ITERATOR_RESET(mi)
+    pi = NI_GetArrayData(array);
+    pm = NI_GetArrayData(mask);
+    for(jj = 0; jj < size; jj++) {
+      int value = *(Int8*)pm;
+      if (value >= 0)
+        *(Bool*)pi = value;
+      NI_ITERATOR_NEXT2(ii, mi,  pi, pm)    
+    }
+  }
+
+ exit:
+  if (offsets) 
+    free(offsets);
+  if (coordinate_offsets)
+    free(coordinate_offsets);
+  NI_FreeCoordinateList(list1);
+  NI_FreeCoordinateList(list2);
+  if (PyErr_Occurred()) {
+    return 0;
+  } else {
+    return 1;
+  }
   return PyErr_Occurred() ? 0 : 1;
 }
 
@@ -561,10 +889,11 @@ int NI_DistanceTransformOnePass(PyArrayObject *strct,
   /* calculate the filter offsets: */
   if (!NI_InitFilterOffsetsFromArray(distances, tmp, shifts, 
                                      NI_EXTEND_CONSTANT, 
-                                     &offsets, &mask_value))
+                                     &offsets, &mask_value, NULL))
     goto exit;
   /* initialize filter iterator: */
-  if (!NI_InitFilterIterator(arank, sdims, filter_size, adims, shifts, &si))
+  if (!NI_InitFilterIterator(arank, sdims, filter_size, adims, shifts,
+                             &si))
     goto exit;
   
   if (features) {
@@ -576,7 +905,7 @@ int NI_DistanceTransformOnePass(PyArrayObject *strct,
     /* calculate the filter offsets: */
     if (!NI_InitFilterOffsetsFromArray(features, tmp, shifts, 
                                        NI_EXTEND_CONSTANT, 
-                                       &foffsets, &dummy))
+                                       &foffsets, &dummy, NULL))
       goto exit;
     /* initialize filter iterator: */
     if (!NI_InitFilterIterator(arank, sdims, filter_size, adims, shifts,
