@@ -169,7 +169,7 @@ int NI_AllocateLineBuffer(PyArrayObject* array, int axis, int size1,
   /* the number of lines of the array is an upper limit for the
      number of lines in the buffer: */
   max_lines = NI_Elements(array);
-  if (adims[axis] > 0 && arank > 0)
+  if (arank > 0 && adims[axis] > 0)
     max_lines /= adims[axis];
   
   /* calculate the space needed for one line, including space to
@@ -615,7 +615,7 @@ int NI_InitFilterIterator(int rank, int *filter_shape, int size,
     iterator->backstrides[ii] = (step - 1) * iterator->strides[ii];
     /* initialize boundary extension sizes: */
     iterator->bound1[ii] = shft;
-    iterator->bound2[ii] = array_shape[ii] - filter_shape[ii] + shft - 1;
+    iterator->bound2[ii] = array_shape[ii] - filter_shape[ii] + shft;
   }
 
   return 1;
@@ -626,7 +626,8 @@ int NI_InitFilterIterator(int rank, int *filter_shape, int size,
 int NI_InitFilterOffsetsFromArray(PyArrayObject *array, 
                                   PyArrayObject *footprint, int* shifts, 
                                   NI_ExtendMode extend_mode, int **offsets, 
-                                  int *border_flag_value)
+                                  int *border_flag_value,
+                                  int **coordinate_offsets)
 {
   int dimensions[NI_MAXDIM], strides[NI_MAXDIM], arank;
 
@@ -637,7 +638,7 @@ int NI_InitFilterOffsetsFromArray(PyArrayObject *array,
   NI_GetArrayStrides(array, strides);
   return NI_InitFilterOffsets(arank, dimensions, strides, footprint,
                               shifts, extend_mode, offsets,
-                              border_flag_value);
+                              border_flag_value, coordinate_offsets);
 }
 
 /* Calculate the offsets to the filter points, for all border regions and
@@ -645,11 +646,11 @@ int NI_InitFilterOffsetsFromArray(PyArrayObject *array,
 int NI_InitFilterOffsets(int rank, int* dimensions, int* strides,
                          PyArrayObject *footprint, int* shifts, 
                          NI_ExtendMode extend_mode, int **offsets, 
-                         int *border_flag_value)
+                         int *border_flag_value, int **coordinate_offsets)
 {
   int filter_size, footprint_size = 0, offsets_size = 1, ii, jj, kk;
-  int *po, shape[NI_MAXDIM], coordinates[NI_MAXDIM], position[NI_MAXDIM];
-  int max_size = 0, max_stride = 0, fdims[NI_MAXDIM];
+  int shape[NI_MAXDIM], coordinates[NI_MAXDIM], position[NI_MAXDIM];
+  int max_size = 0, max_stride = 0, fdims[NI_MAXDIM], *po, *pc = NULL;
   Bool *pm;
   
   assert(dimensions != NULL);
@@ -713,6 +714,14 @@ int NI_InitFilterOffsets(int rank, int* dimensions, int* strides,
     PyErr_NoMemory();
     goto exit;
   }
+  if (coordinate_offsets) {
+    *coordinate_offsets = (int*)malloc(offsets_size * rank * 
+                                       footprint_size * sizeof(int));
+    if (!*coordinate_offsets) {
+      PyErr_NoMemory();
+      goto exit;
+    }
+  }
 
   for(ii = 0; ii < rank; ii++) {
     int stride;
@@ -735,6 +744,9 @@ int NI_InitFilterOffsets(int rank, int* dimensions, int* strides,
   /* calculate all possible offsets to elements in the filter kernel, 
      for all regions in the array (interior and border regions): */
   po = *offsets;
+  if (coordinate_offsets) {
+    pc = *coordinate_offsets;
+  }
   /* iterate over all regions: */
   for(jj = 0; jj < offsets_size; jj++) {
     /* iterate over the elements in the footprint array: */
@@ -828,16 +840,22 @@ int NI_InitFilterOffsets(int rank, int* dimensions, int* strides,
           if (cc == *border_flag_value) {
             /* just flag that we are outside the border */
             offset = *border_flag_value;
+            if (coordinate_offsets)
+              pc[ii] = 0;
             break;
           } else {
             /* use an offset that is possibly mapped from outside the
                border: */
             cc = cc - position[ii];
             offset += strides[ii] * cc;
+            if (coordinate_offsets)
+              pc[ii] = cc;
           }
         }
         /* store the offset */
         *po++ = offset;
+        if (coordinate_offsets)
+          pc += rank;
       }
       /* next point in the filter: */
       for(ii = rank - 1; ii >= 0; ii--) {
@@ -867,14 +885,102 @@ int NI_InitFilterOffsets(int rank, int* dimensions, int* strides,
       }
     }
   }
-
+  
  exit:
   if (PyErr_Occurred()) {
     if (*offsets)
       free(*offsets);
+    if (coordinate_offsets && *coordinate_offsets)
+      free(*coordinate_offsets);
     return 0;
   } else {
     return 1;
   }
 }
 
+NI_CoordinateList* NI_InitCoordinateList(int size, int rank)
+{
+  NI_CoordinateList *list = \
+    (NI_CoordinateList*)malloc(sizeof(NI_CoordinateList));
+  if (!list) {
+    PyErr_NoMemory();
+    return NULL;
+  }
+  list->block_size = size;
+  list->rank = rank;
+  list->blocks = NULL;
+  return list;
+}
+
+int NI_CoordinateListStealBlocks(NI_CoordinateList *list1, 
+                                 NI_CoordinateList *list2)
+{
+  if (list1->block_size != list2->block_size ||
+      list1->rank != list2->rank) {
+    PyErr_SetString(PyExc_RuntimeError, "coordinate lists not compatible");
+    return 1;
+  }
+  if (list1->blocks) {
+    PyErr_SetString(PyExc_RuntimeError, "first is list not empty");
+    return 1;
+  }
+  list1->blocks = list2->blocks;
+  list2->blocks = NULL;
+  return 0;
+}
+
+NI_CoordinateBlock* NI_CoordinateListAddBlock(NI_CoordinateList *list)
+{
+  NI_CoordinateBlock* block = NULL;
+  block = (NI_CoordinateBlock*)malloc(sizeof(NI_CoordinateBlock));
+  if (!block) {
+    PyErr_NoMemory();
+    goto exit;
+  }
+  block->coordinates = (int*)malloc(list->block_size * list->rank * 
+                                    sizeof(int));
+  if (!block->coordinates) {
+    PyErr_NoMemory();
+    goto exit;
+  }
+  block->next = list->blocks;
+  list->blocks = block;
+  block->size = 0;
+  
+exit:
+  if (PyErr_Occurred()) {
+    if (block)
+      free(block);
+    return NULL;
+  }
+  return block;
+}
+
+NI_CoordinateBlock* NI_CoordinateListDeleteBlock(NI_CoordinateList *list)
+{
+  NI_CoordinateBlock* block = list->blocks;
+  if (block) {
+    list->blocks = block->next;
+    if (block->coordinates) 
+      free(block->coordinates);
+    free(block);
+  }
+  return list->blocks;
+}
+
+void NI_FreeCoordinateList(NI_CoordinateList *list)
+{
+  if (list) {
+    NI_CoordinateBlock *block = list->blocks;
+    while (block) {
+      NI_CoordinateBlock *tmp = block;
+      block = block->next;
+      if (tmp->coordinates)
+        free(tmp->coordinates);
+      free(tmp);
+    }
+    list->blocks = NULL;
+    free(list);
+  }
+}
+
