@@ -31,6 +31,13 @@ _ndarray_rank_get(PyArrayObject *self)
 	return PyInt_FromLong(self->nd);
 }
 
+static PyObject *
+_ndarray_flags_get(PyArrayObject *self)
+{
+	NA_updateStatus(self);
+	return PyInt_FromLong(self->flags);
+}
+
 static PyObject *
 _ndarray_shape_get(PyArrayObject *self)
 {
@@ -196,6 +203,8 @@ _ndarray_byteoffset_set(PyArrayObject *self, PyObject *s)
 		PyErr_Format(PyExc_TypeError, "_byteoffset must be an integer.");
 		rval = -1;
 	}
+	if (!NA_updateDataPtr(self))
+		return -1;
 	NA_updateStatus(self);
 	return rval;
 }
@@ -207,6 +216,8 @@ Setting _aligned *defines* an array's alignment state.
 static PyObject *
 _ndarray_aligned_get(PyArrayObject *self)
 {
+	if (self->_aligned == MUST_BE_COMPUTED)
+		return NULL;
 	return PyInt_FromLong(self->_aligned);
 }
 
@@ -273,6 +284,10 @@ static PyGetSetDef _ndarray_getsets[] = {
 	 (getter)_ndarray_rank_get, 
 	 (setter) 0, 
 	 "tuple of array dimensions"}, 
+	{"flags", 
+	 (getter)_ndarray_flags_get, 
+	 (setter) 0, 
+	 "integer value of flags"}, 
 	{"_shape", 
 	 (getter)_ndarray_shape_get, 
 	 (setter)_ndarray_shape_set, 
@@ -404,17 +419,17 @@ _ndarray_init(PyArrayObject *self, PyObject *args, PyObject *kwds)
 	if (buffer != Py_None) {
 		self->_data = buffer;
 		Py_INCREF(buffer);
-		if (!NA_updateDataPtr(self))
-			return -1;
 	} else {
 		Int64 size = self->bytestride * 
 			(Int64) NA_elements(self);
 		self->_data = PyObject_CallFunction(
 			pNewMemoryFunc, "(L)", size);
-		if (!self->_data || !NA_updateDataPtr(self)) 
-			return -1;
+		if (!self->_data) return -1;
 	}
 	
+	if (!NA_updateDataPtr(self))
+		return -1;
+
 	_stridesFromShape(self);
 	self->_shadows = NULL;
 	NA_updateStatus(self);
@@ -480,8 +495,8 @@ _ndarray_setitem(PyArrayObject *self, long offset, PyObject *value)
 	int rval = -1;
 	
 	if (value == NULL) {
-		PyErr_Format(PyExc_RuntimeError,
-			     "_ndarray_setitem: can't delete from an array");
+		PyErr_Format(PyExc_TypeError,
+			     "_ndarray_setitem: can't delete elements from an array");
 		return -1;
 	}
 
@@ -529,6 +544,8 @@ _simpleIndexingCore(PyArrayObject *self, long offset, int nindices, PyObject *va
 		}
 		
 		result->byteoffset = offset;
+		if (!NA_updateDataPtr(result))
+			return NULL;
 
 		NA_updateStatus(result);
 
@@ -639,11 +656,14 @@ _snip( int narr, maybelong *arr, int i)
 static PyObject *
 _slicedIndexing0(PyArrayObject *self, PyObject *key, PyObject *value, int dim)
 {
-	PyObject *result;
+	PyObject *result=NULL;
 	int keylen = PyList_Size(key);
 	if (keylen < 0) return NULL;
 
 	if (keylen == 0) {
+		if (!NA_updateDataPtr(self)) {
+			return NULL;
+		}
 		if (value == Py_None) { /* get slice */
 			NA_updateStatus(self);
 			result = (PyObject *) self;
@@ -983,6 +1003,14 @@ static int
 _ndarray_ass_subscript(PyArrayObject * self, PyObject* key, PyObject* value)
 {
 	PyObject *result;
+
+	if (!value) {
+		PyErr_Format(
+			PyExc_TypeError, 
+			"_ndarray_ass_subscript: can't delete elements from an array.");
+		return -1;
+	}
+
 #if _PYTHON_CALLBACKS
 	result = PyObject_CallMethod(
 		(PyObject *) self, "_universalIndexing", "(OO)", key, value);
@@ -1039,6 +1067,9 @@ _ndarray_isaligned(PyArrayObject *self, PyObject  *args)
 {
 	if (!PyArg_ParseTuple(args, ":isaligned"))
 		return NULL;
+	if (!NA_updateDataPtr(self))
+		return NULL;
+	NA_updateAlignment(self);
 	return PyInt_FromLong((self->flags & ALIGNED) != 0);
 }
         
@@ -1103,11 +1134,6 @@ _view(PyArrayObject *self)
 
 	if (PyDict_Update(vd, sd) < 0) goto _fail;
 
-	Py_XDECREF(view->_data);
-	view->_data = self->_data;  
-	Py_INCREF(self->_data);
-	if (!NA_updateDataPtr(self)) goto _fail;
-
 	for(i=0; i<self->nd; i++)
 		view->dimensions[i] = self->dimensions[i];
 	view->nd = self->nd;
@@ -1125,6 +1151,13 @@ _view(PyArrayObject *self)
 	/* Numarray hack here; since fields are always present, copy them. */
 	view->descr = self->descr;
 	view->byteorder = self->byteorder;
+
+	Py_XDECREF(view->_data);
+	view->_data = self->_data;  
+	Py_INCREF(self->_data);
+	if (!NA_updateDataPtr(view)) goto _fail;
+
+	NA_updateStatus(view);
 	goto _exit;
 
   _fail:
@@ -1222,7 +1255,7 @@ _taker(PyArrayObject *self, int dim,
 	if (dim == indices->nd - 1) {
 		PyObject *obj, *ans;
 		long selfoff;
-		char *index = (char *) NA_OFFSETDATA(indices);
+		char *index = (char *) indices->data;
 		index += indoff;
 		if (NA_getByteOffset(
 			    self, dimlen, (maybelong *) index, &selfoff) < 0)
@@ -1284,7 +1317,7 @@ _putter(PyArrayObject *self, int dim,
 	if (dim == indices->nd - 1) {
 		PyObject *obj, *ans;
 		long selfoff;
-		char *index = (char *) NA_OFFSETDATA(indices);
+		char *index = (char *) indices->data;
 		index += indoff;
 		if (NA_getByteOffset(
 			    self, dimlen, (maybelong *) index, &selfoff) < 0)
@@ -1427,6 +1460,22 @@ _ndarray_product(PyObject *module, PyObject *args)
 	return Py_BuildValue("i", prod);
 }
 
+static PyObject *
+_ndarray_lp64(PyObject *module, PyObject *args)
+{
+	if (!PyArg_ParseTuple(args, ":lp64"))
+		return NULL;
+	return PyInt_FromLong(LP64);
+}
+
+static PyObject *
+_ndarray_hasUInt64(PyObject *module, PyObject *args)
+{
+	if (!PyArg_ParseTuple(args, ":hasUInt64"))
+		return NULL;
+	return PyInt_FromLong(HAS_UINT64);
+}
+
 static PyMethodDef 
 _ndarray_functions[] = {
 	{"_isIntegerSequence", 
@@ -1437,6 +1486,14 @@ _ndarray_functions[] = {
 	 (PyCFunction) _ndarray_product,
 	 METH_VARARGS,
 	 "product(t)   returns the product of all elements of an integer sequence"},
+	{"lp64", 
+	 (PyCFunction) _ndarray_lp64,
+	 METH_VARARGS,
+	 "lp64()  returns flag indicating whether this system and compiler use the 64-bit LP64 model."},
+	{"hasUInt64", 
+	 (PyCFunction) _ndarray_hasUInt64,
+	 METH_VARARGS,
+	 "hasUInt64()  returns flag indicating whether this system has 64-bit unsigned integer support."},
 	{NULL,		NULL}		/* sentinel */
 };
 
@@ -1458,8 +1515,6 @@ init_ndarray(void)
 	_ndarray_type.tp_base = NULL;
 	_ndarray_type.tp_alloc = PyType_GenericAlloc;
 
-	import_libnumarray();
-
 	if (PyType_Ready(&_ndarray_type) < 0)
 		return;
 
@@ -1479,6 +1534,8 @@ init_ndarray(void)
 	if (PyModule_AddObject(m, "_alignment", _alignment) < 0)
 		return;
 
+	import_libnumarray();
+
 	pNewMemoryFunc = NA_initModuleGlobal("numarray.memory","new_memory");
 	if (!pNewMemoryFunc)
 		PyErr_Format(
@@ -1496,4 +1553,6 @@ init_ndarray(void)
 	if (!NullArgs) return;
 	NullKeywds = PyDict_New();
 	if (!NullKeywds) return;
+
 }
+
