@@ -1,4 +1,4 @@
-/* $Id: _numarraymodule.c,v 1.55 2004/06/24 15:33:35 jaytmiller Exp $ */
+/* $Id: _numarraymodule.c,v 1.62 2004/11/01 15:12:29 jaytmiller Exp $ */
 
 #include <Python.h>
 #include <stdio.h>
@@ -354,6 +354,7 @@ _copyFrom(PyObject *self, PyObject *arr0)
 	if (!barr) return NULL;
 	result = PyObject_CallFunction(
 		p_copyFromAndConvert, "(OO)", barr, self);
+	if (!result) return NULL;
 	Py_DECREF(barr);
 	return result;
 }
@@ -524,10 +525,12 @@ static PyArrayObject *
 _rank0_to_rank1(PyArrayObject *ao)
 {
 	PyArrayObject *bo;
-	if (ao->nd != 0) return ao;
-
+	if (ao->nd != 0) {
+		Py_INCREF(ao);
+		return ao;
+	}
 	bo = NA_copy(ao);
-	Py_DECREF(ao);
+	if (!bo) return NULL;
 
 	bo->dimensions[0] = 1;
 	bo->nd = 1;
@@ -544,6 +547,7 @@ _innerproduct(PyArrayObject *a, PyArrayObject *b, NumarrayType maxt)
 
 	a = _rank0_to_rank1(a);
 	b = _rank0_to_rank1(b);
+	if (!a || !b) return NULL;
 
 	/* Result shape =  a.shape[:-1] + b.shape[:-1] */
 	for(i=0; i<a->nd-1; i++)
@@ -552,6 +556,12 @@ _innerproduct(PyArrayObject *a, PyArrayObject *b, NumarrayType maxt)
 		dots[i] = b->dimensions[i-a->nd+1];
 
 	r = NA_vNewArray(NULL, maxt, a->nd + b->nd - 2, dots);
+	if (!r)
+	{
+	  Py_DECREF(a);
+	  Py_DECREF(b);
+	  return NULL;
+	}
 
 	imax = NA_elements(a) / a->dimensions[ a->nd - 1];
 	jmax = NA_elements(b) / b->dimensions[ b->nd - 1];
@@ -559,9 +569,9 @@ _innerproduct(PyArrayObject *a, PyArrayObject *b, NumarrayType maxt)
 
 	NA_clearFPErrors();
 
-	ap = NA_OFFSETDATA(a);
-	bp = NA_OFFSETDATA(b);
-	rp = NA_OFFSETDATA(r);
+	ap = a->data;
+	bp = b->data;
+	rp = r->data;
 
 	switch((int) maxt) 
 	{
@@ -620,79 +630,103 @@ static PyObject *
 innerproduct(PyObject *module, PyObject *args)
 {
 	PyObject *ao, *bo;
-	PyArrayObject *a, *b;
+	PyArrayObject *a=NULL, *b=NULL;
+	PyObject  *result=NULL;
 	NumarrayType at, bt, maxt;
 
 	if (!PyArg_ParseTuple(args, "OO:innerproduct", &ao, &bo))
-		return NULL;
+		goto _done;
 	
 	if ((at = _dot_type(ao)) < 0)
-		return NULL;
+		goto _done;
 
 	if ((bt = _dot_type(bo)) < 0)
-		return NULL;
+		goto _done;
 	
 	maxt = MAX(at, bt);
 
 	if (maxt == tBool) maxt = tLong;
 	
 	a = NA_InputArray(ao, maxt, NUM_C_ARRAY);
-	if (!a) return NULL;
+	if (!a) goto _done;
 
 	b = NA_InputArray(bo, maxt, NUM_C_ARRAY);
-	if (!b) return NULL;
+	if (!b) goto _done;
 
 	if (a->dimensions[a->nd-1] != b->dimensions[b->nd-1])
-		return PyErr_Format(
+	{
+		PyErr_Format(
 			PyExc_ValueError, 
 			"innerproduct: last sequence dimensions must match.");
-	
-	return _innerproduct(a,b,maxt);
+		goto _done;
+	}
+
+	result = _innerproduct(a,b,maxt);
+
+  _done:
+	Py_XDECREF(a);
+	Py_XDECREF(b);
+	return result;
 }
 
 static PyObject *
 dot(PyObject *module, PyObject *args)
 {
 	PyObject *ao, *bo;
-	PyArrayObject *a, *b;
+	PyArrayObject *a=NULL, *b=NULL, *b_original=NULL, *b2=NULL;
 	NumarrayType at, bt, maxt;
+	PyObject *result=NULL;
 
 	if (!PyArg_ParseTuple(args, "OO:dot", &ao, &bo))
-		return NULL;
-	
-	
+		goto _done;
+
 	if ((at = _dot_type(ao)) < 0)
-		return NULL;
+		goto  _done;
 
 	if ((bt = _dot_type(bo)) < 0)
-		return NULL;
-	
+		goto _done;
+
 	maxt = MAX(at, bt);
 	
 	a = NA_InputArray(ao, maxt, NUM_C_ARRAY);
-	if (!a) return NULL;
+	if (!a) goto _done;
 
 	b = NA_InputArray(bo, maxt, NUM_C_ARRAY);
-	if (!b) return NULL;
+	if (!b) goto _done;
 
-	if (NA_swapAxes(b, -1, -2) < 0) {
-		Py_DECREF(a);
-		Py_DECREF(b);
-		return NULL;
+	if (NA_swapAxes(b, -1, -2) < 0)
+		goto _done;
+
+	b_original = b;
+	Py_INCREF(b_original);
+	if (!PyArray_ISCARRAY(b)) {
+		b2 = NA_copy(b);
+	} else {
+		b2 = b;
+		Py_INCREF(b);
 	}
 
-	if (!PyArray_ISCARRAY(b)) {
-		PyArrayObject *b2 = b;
-		b = NA_copy(b);
-		Py_DECREF(b2);
-	}		
-
-	if (a->dimensions[a->nd-1] != b->dimensions[b->nd-1])
-		return PyErr_Format(
-			PyExc_ValueError, 
+	if (a->dimensions[a->nd-1] != b2->dimensions[b2->nd-1]) {
+		NA_swapAxes(b_original, -1, -2);
+		PyErr_Format(PyExc_ValueError,
 			"innerproduct: last sequence dimensions must match.");
-	
-	return _innerproduct(a,b,maxt);
+		goto _done;
+	}
+
+	result = _innerproduct(a,b2,maxt);
+
+	if (NA_swapAxes(b_original, -1, -2) < 0) {
+		Py_DECREF(result);
+		result = NULL;
+		goto _done;
+	}
+
+  _done:
+	Py_XDECREF(a);
+	Py_XDECREF(b);
+	Py_XDECREF(b_original);
+	Py_XDECREF(b2);
+	return result;
 }
 
 
