@@ -260,6 +260,10 @@ typedef struct {
 	int    arity;
 	ufunc_cache cache;
 } PyUfuncObject;
+
+#if defined(UNTHREADED)
+static long PyThread_get_thread_ident() { return 0; }
+#endif
  
 static void 
 _cache_insert(ufunc_cache *cache, PyObject *ctuple,
@@ -552,6 +556,7 @@ _accumulate_out(PyObject *self, PyObject *inarr, PyObject *outarr, PyObject *oty
 		Py_INCREF(toutarr);
 	}
 
+	assert(NA_NumArrayCheck(inarr));
 	if (NA_elements((PyArrayObject *)inarr)) {
 		fc_in = _firstcol(inarr);
 		if (inarr != toutarr)
@@ -593,6 +598,7 @@ _reduce_out(PyObject *self, PyObject *inarr, PyObject *outarr, PyObject *otype)
 		PyExc_ValueError, "_reduce_out: problem with otype");
 
 	/* because _firstcol might drop zero dimension compute now. */
+	assert(NA_NumArrayCheck(inarr));
 	nelements = NA_elements((PyArrayObject *)inarr); 
 
 	fc_in = _firstcol(inarr);
@@ -759,6 +765,7 @@ _cum_slow_exec(PyObject *self, PyObject *in1, PyObject *out, PyObject *cached)
 	if (!in1_shape) return NULL;
 
 	blocking = _getBlockingParameters(in1_shape, niter, overlap);
+	Py_DECREF(in1_shape);
 	
 	if (!PyArg_ParseTuple(blocking, "iO:_cum_slow_exec blocking", 
 			      &indexlevel, &blockingparameters)) {
@@ -798,6 +805,7 @@ _cum_slow_exec(PyObject *self, PyObject *in1, PyObject *out, PyObject *cached)
 #else
 	result = _callOverDimensions(objects, oshape, indexlevel, blockingparameters, overlap, 0);
 #endif
+	Py_DECREF(oshape);
 	if (!result) return NULL;
 
  	inbuff = ConverterRebuffer(input, Py_None, Py_None);
@@ -807,6 +815,7 @@ _cum_slow_exec(PyObject *self, PyObject *in1, PyObject *out, PyObject *cached)
   	if (!outbuff) return NULL;
 	Py_DECREF(outbuff);
 	Py_DECREF(objects);
+	Py_DECREF(blocking);
 	return result;
 }
 
@@ -839,7 +848,10 @@ _cum_exec(PyObject *self, PyObject *in1, PyObject *out, PyObject *cached)
 	if ((otype == tBool) && (in1a->descr->type_num != tBool)) {
 		in1 = PyObject_CallMethod(in1, "astype", "(s)", "Bool");
 		if (!in1) return NULL;
+	} else {
+		Py_INCREF(in1);
 	}
+	assert(NA_NumArrayCheck(in1));
 	if (NA_elements((PyArrayObject *)in1)) {
 		NA_clearFPErrors();
 		mode = PyString_AsString(modeObj);
@@ -860,11 +872,14 @@ _cum_exec(PyObject *self, PyObject *in1, PyObject *out, PyObject *cached)
 			result = _cum_slow_exec(self, in1, out, cached);
 #endif
 		}
-		if (_reportErrors(self, result, out) < 0)
+		if (_reportErrors(self, result, out) < 0) {
+			Py_DECREF(in1);
 			return NULL;
+		}
 	} else {
 	} /* 0-element case doesn't require ufunc */
 	Py_INCREF(out);
+	Py_DECREF(in1);
 	return out;
 }
 
@@ -1651,20 +1666,26 @@ _cache_exec1(PyObject  *ufunc, PyObject *in1, PyObject *out,
 {
 	PyObject *result, *mode;
 	char *mode_str;
-
-	NA_clearFPErrors();
-	mode = PyTuple_GET_ITEM(cached, 0);
-	if (!PyString_Check(mode))
-		return PyErr_Format(PyExc_ValueError, 
-				    "_cache_exec1: mode is not a string");
-	mode_str = PyString_AS_STRING(mode);
-	if (!strcmp(mode_str, "fast")) {
-		result = _fast_exec1(ufunc, in1, out, cached);
-	} else {
-		result = _slow_exec1(ufunc, in1, out, cached);
+	long count;	
+	assert(NA_NumArrayCheck(out));
+	count = NA_elements((PyArrayObject *) out);
+	if (count < 0) return NULL;
+	if (count) {
+		NA_clearFPErrors();
+		mode = PyTuple_GET_ITEM(cached, 0);
+		if (!PyString_Check(mode))
+			return PyErr_Format(
+				PyExc_ValueError, 
+				"_cache_exec1: mode is not a string");
+		mode_str = PyString_AS_STRING(mode);
+		if (!strcmp(mode_str, "fast")) {
+			result = _fast_exec1(ufunc, in1, out, cached);
+		} else {
+			result = _slow_exec1(ufunc, in1, out, cached);
+		}
+		if (_reportErrors(ufunc, result, out) < 0)
+			return NULL;
 	}
-	if (_reportErrors(ufunc, result, out) < 0)
-		return NULL;
 	return _returnScalarOrArray(out);
 }
 
@@ -1967,24 +1988,31 @@ _cache_exec2(PyObject  *ufunc, PyObject *in1, PyObject *in2, PyObject *out,
 {
 	PyObject *result, *mode;
 	char *mode_str;
+	long count;
 
-	NA_clearFPErrors();
-	mode = PyTuple_GET_ITEM(cached, 0);
-	if (!PyString_Check(mode))
-		return PyErr_Format(PyExc_ValueError, 
-				    "_cache_exec2: mode is not a string");
-	mode_str = PyString_AS_STRING(mode);
-	if (!strcmp(mode_str, "fast")) {
-		tc_start_clock("_cache_exec2 fast");
-		result = _fast_exec2(ufunc, in1, in2, out, cached);
-		tc_stop_clock("_cache_exec2 fast");
-	} else {
-		tc_start_clock("_cache_exec2 slow");
-		result = _slow_exec2(ufunc, in1, in2, out, cached);
-		tc_stop_clock("_cache_exec2 slow");
+	assert(NA_NumArrayCheck(out));
+	count = NA_elements((PyArrayObject *)out);
+	if (count < 0) return NULL;
+	if (count) {
+		NA_clearFPErrors();
+		mode = PyTuple_GET_ITEM(cached, 0);
+		if (!PyString_Check(mode))
+			return PyErr_Format(
+				PyExc_ValueError, 
+				"_cache_exec2: mode is not a string");
+		mode_str = PyString_AS_STRING(mode);
+		if (!strcmp(mode_str, "fast")) {
+			tc_start_clock("_cache_exec2 fast");
+			result = _fast_exec2(ufunc, in1, in2, out, cached);
+			tc_stop_clock("_cache_exec2 fast");
+		} else {
+			tc_start_clock("_cache_exec2 slow");
+			result = _slow_exec2(ufunc, in1, in2, out, cached);
+			tc_stop_clock("_cache_exec2 slow");
+		}
+		if (_reportErrors(ufunc, result, out) < 0)
+			return NULL;
 	}
-	if (_reportErrors(ufunc, result, out) < 0)
-		return NULL;
 	return _returnScalarOrArray(out);
 }
 
